/*
**  Copyright (C) 1993-1996, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/

/*
**			ctndsupp.cc
**
**	This file contains routines in support of program CTNDISP.
**
**	int	CTNDISP_LoadGrayMap (int n)
**		A recursive routine to allocate as many as 240 gray-scale
**		levels.  If the initial allocation is unsuccessful the
**		routine returns the number of levels that were successfully 
**		allocated.  A gray-scale look up table is loaded with
**		the numbers of the cells that were successfully allocated.
**	
**	int CTNDISP_InitializeX (int *ncolors)
**		This routine connects to the X server and retrieves
**		various attributes for the current screen.  It also
**		retrieves three different graphics contexts used in CTNDISP.
**		The gray-scale color table is loaded and if fewer
**		than 16 levels are allocated, an error is returned.
**		A window encompassing the entire display is created
**		and mapped. 
**	
**	void	CTNDISP_DisplayImageDataS(  )
**		Displays the actual image data to the screen at position
**		in the image structs.  A window is created and an XImage structure is
**		created and "put" to the screen.  	
**
**	
**	unsigned long CTNDISP_CreateGC ( void )
**		This routine creates several different graphics contexts
**		used in CTNDISP.  If the two fonts that are required
**		cannot be loaded, a fatal error is returned.  If any
**		of the graphics contexts cannot be loaded, a fatal
**		error is returned.  
**
**	void	CTNDISP_RefreshImageData( ImageStruct *img, XImageStruct *ximg )
**		Simple routine to refresh the image data and associated text 
**		after a window move event or an expose event.
**
**	void	CTNDISP_RefreshStatus()
**
**	
** ctndsupp.cc,v
** Revision 1.3  1996/10/18 14:32:29  hewett
** Added cast to (char*) for 2nd argument of call to shmat(...).
** Requested by Larry V. Streepy, Jr to eliminate compiler warning.
**
** Revision 1.2  1996/04/18 15:29:41  meichel
** Version for CAR'96
**
** Revision 1.1  1996/03/28 17:06:37  meichel
** first commit with new file structure
**
** Revision 2.4  1995/05/19 14:49:18  hewett
** Aspect ratio is now preserved in image display.  Previously all images
** were made square.  Image now centered within displayable area.
**
 * Revision 2.2  1995/03/01  14:26:53  meichel
 * Fixed grayscale LUT problem. Now works correctly with Solaris 2.
 *
 * Revision 2.1  1995/02/02  12:49:27  hewett
 * Corrected spelling of Ossietzky
 *
 * Revision 2.0  1994/11/17  10:12:01  meichel
 * Version 2.0 for RNSA 1994 demonstration
 *
 * Revision 1.30  1994/11/17  09:44:10  meichel
 * Version 2.0 for RSNA 1994 demonstration.
 * New features: Metaheader support, new SOP classes, lots of bugs fixed.
 *
 * Revision 1.29  1994/03/17  18:15:16  hewett
 * Changes due to must recent DICOM data dictionary (dcmcode)
 *
 * Revision 1.28  1994/03/17  09:27:47  hewett
 * Updated copyright notice.
 *
 * Revision 1.27  1993/11/22  16:07:14  jens
 * Made Imagearea really square, but it`s hip ..
 *
 * Revision 1.26  1993/11/22  12:33:04  hewett
 * Made image queue a linked list (was using 10mb static memory!!)
 *
 * Revision 1.25  1993/11/19  15:11:37  jens
 * Fixed some bugs.
 *
 * Revision 1.24  1993/11/18  20:17:05  jens
 * Multi processes Version of ctndisp.
 * If you don`t want forked image conversion, then use -noMulti switch
 *
 * Revision 1.23  1993/11/16  12:24:35  jens
 * Added switch -2images to ctndisp. Only 2 Imageareas are used.
 * Modified the Assiciation window refresh. (less flicker)
 * Added a image count information.
 * Fasten XputImage.
 * Added code for the killassoc command.
 *
 * Revision 1.22  1993/11/11  15:02:33  jens
 * Made a unsteady free() command safe.
 *
 * Revision 1.21  1993/11/10  22:37:30  jens
 * removed a unused function. Fixed a bug in GetElement.
 *
 * Revision 1.19  1993/11/10  17:42:58  jens
 * Added a new flag: -half
 * with ctndisp -half the window uses a square half of the screen
 *
 * Revision 1.18  1993/11/04  15:57:25  jens
 * -noRescale and -noWindow switches added. see README.
 *
 * Revision 1.17  1993/11/02  14:59:48  jens
 * removed a TILT debug info, leck of noise. ?
 *
 * Revision 1.15  1993/11/01  21:02:19  jens
 * Better error resistent Version in Tag reading section.
 *
 * Revision 1.14  1993/10/29  15:27:56  jens
 * With Rescale Intercept & Slope, Window Level calculation.
 * One Image seems to be corrupt, because calculation is correct, but it is
 * shown strange. But don`t you eat the yellow snow, watch out where the huskies
 * go. Nothig is really perfect, but everything goes.
 *
 * Revision 1.13  1993/10/26  16:05:29  jens
 * Included DSU_displayIndependant command for sending w/o association
 *
 * Revision 1.12  1993/10/20  10:16:43  jens
 * CTNDISPLAY with better Palettecolor conversion
 *
 * Revision 1.11  1993/10/14  10:46:11  hewett
 * Added Euro Header
 *
 * Revision 1.10  1993/10/14  10:16:43  hewett
 * Malloc was being called with two parameters.  Changed such calls to calloc.
 *
**
** ctndsupp.cc,v 1.3 1996/10/18 14:32:29 hewett Exp
**
*/

#include "osconfig.h" /* configure mechanism */

#ifndef X_DISPLAY_MISSING
#if defined(HAVE_SHMAT) && defined(HAVE_SHMCTL) && defined(HAVE_SHMGET)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern "C"
{
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_IPC_H
#include <sys/ipc.h>
#endif

#ifdef HAVE_SYS_SHM_H
#include <sys/shm.h>
#else
/* 
   This is a dummy declaration of struct shmid_ds. The real declaration from 
   sys/shm.h is very complex. Since we only pass "opaque" pointers of this 
   type to shmctl() and don't need the results, this hack should work.
   Anyway: Which weard kind of system
   should support shmctl() and have no appropriate header file??? 
 */
     struct shmid_ds { int dummy_element; };
#endif

#ifndef HAVE_PROTOTYPE_SHMAT
     void *shmat(int shmid, void *shmaddr, int shmflg);
#endif
#ifndef HAVE_PROTOTYPE_SHMCTL
     int shmctl(int shmid, int cmd, struct shmid_ds *buf);
#endif
#ifndef HAVE_PROTOTYPE_SHMGET
     int shmget(key_t key, int size, int shmflg);
#endif

}

#include "snq.h"	
#include "disppriv.h"	
#include "ctnddico.h"
#include "ctndsupp.h"

#define TMPFNAME "TmpObj.dcm";
#define  NODIRECTION " -- "
#define  RIGHTDIRECTION "--> "
#define  LEFTDIRECTION " <--"

#ifndef MAX
#define MAX(x, y) (((x) < (y)) ? (y) : (x))
#endif
#ifndef MIN
#define MIN(x, y) (((x) < (y)) ? (x) : (y))
#endif

typedef struct	_XImageStruct{
	XImage	*x_image;	/* the X image struct		*/
	Window	image_window;	/* the window for display	*/
} XImageStruct;

unsigned long		graylut [256];
unsigned short  	graylutoffset; /* thats new replaces the 40 in ConvertImageDate*/
char			*display_name = NULL;

/*
 * These variables used mostly for the status buttons.
 */

Window active_window_IN, active_window_OUT, counter_window_IN, counter_window_OUT;

static int show_width = 340, 
	show_height = 36, 
	text_height = 0, 
	image_text_height = 0, 
	image_text_offset = 0, 
	SHOW_X_OFFSET = 330,	/* offset of node windows */
 	LINE_OFFSET = 30,	/* percentage line offset in node window */
 	IMAGESWIDTH = 300,	/* dimension of IMAGES */
 	IMAGESHEIGHT = 300,	/* dimension of IMAGES */
 	IMAGEWINDOW1X = 0,	/* LOCATION set in CTNDISP_InitializeX */
 	IMAGEWINDOW1Y = 0, 
	IMAGEWINDOW2X = 0, 
	IMAGEWINDOW2Y = 0, 
	IMAGEWINDOW3X = 0, 
	IMAGEWINDOW3Y = 0, 
	IMAGEWINDOW4X = 0, 	
	IMAGEWINDOW4Y = 0, 
	LINE_THICKNESS = 0, 
	IMAGES = 4,		/* 4 four windows for images */
 	MAXWINDOWS = 20;	/* 20 number of node windows */



XFontStruct *fontSM, *fontLG, *fontSMbold;

#define			BUT_SIZE_PERCT		.14
#define			BUT_SIZE_OFFSET		10
#define			NODE_X_OFFSET		10
#define			NODE_Y_OFFSET		30
#define			SHOW_Y_OFFSET		40
#define			SHOW_SPACE		2	/* space between node
							 * displays */

#define			FREE			-1
#define			TEXTHEIGHT		13	/* pixel height of font */
#define			IMAGEACTIVE		 1
#define			IMAGEINACTIVE		 2

#define			IMAGE_X_OFFSET		3	/* OFFSET of IMAGE
							 * Windows */
#define			IMAGE_Y_OFFSET		40
#define			IMAGE_TEXT_OFFSET	42
#define			IMAGE_SHOW_SPACE	10	/* space between IMAGES
							 * and NODE display
							 * areas */


/*
** Global variables for the X Server
*/

extern int G_display_width, G_display_height, G_screen, G_ncolors;
extern GC G_gclargeT, G_gcsmallT, G_gcsmallTbold, G_gcblack, G_gcred, G_gcgreen;

extern Display *G_display;

extern Window G_big_window;

extern Visual *G_visual;

extern Colormap G_cmap;
extern QUEUE_Element Ctndisplayhandle, CalcElement;	/* CalcElement is the
							 * Image element in
							 * calculation (if
							 * MULTI) */

extern int Rescaleflag, Windowflag;
extern int MULTI;

extern int ImageQueueStatus;
extern int CalcStatus;
extern Image_Modul tempimage;


extern unsigned char *ishmaddr, *tishmaddr;	/* shared memory adress
						 * pointer */
extern int status, died, pid, shmflag;	/* status of son */
extern struct shmid_ds *shm_buf;
extern key_t ishmkey;
extern int ishmsize, ishmflag, ishmID;


/*
 * Other globals...
 */
extern int G_FirstTime;
extern char G_nodename[MAX_STRING_LENGTH];

/*
 * End globals...
 */

struct Sshow_list {
    QUEUE_Element element;
    char show_strings[30][MAX_STRING_LENGTH];	/* 30 = MAXWINDOWS a drawtext
						 * field hold for Window
						 * Refresh event */
    Window show_window;
    int imagecounter;		/* counts the with STORE transferred images */

};

static struct Sshow_list show_list[30];	/* 30 = MAXWINDOWS in this list every
					 * vendor asscociation is connected
					 * to a draw window */


/* the #defines in disppriv.h are mapped to these strings! */
static char *Q_String[] = {
    "", "CTN", "TI",
    "Requestor", "Acceptor", "Main", "Sub",
    "Store", "Find", "Move", "Print", "ImageOp", "InitOp",
    "Start", "Running", "Stop", "In", "Out", "NoAssociation", "KillOperation" };

#define NSQIMAGES
#define NSQCENTERIMAGE
/*** #define NSQCLEARIMAGE ***/
/*** #define DEBUG_IMAGE ***/

/* this datastructure is used for the (static) imageslist */
typedef struct _Images_M {
    unsigned char *image;	/* pointer tospace for converted images , */
#ifdef NSQIMAGES
    int nrows, ncols;		/* number of rows & cols in the reduced image */
#endif
    char AE_Titles[MAX_STRING_LENGTH];
    char Patname[MAX_STRING_LENGTH];
    char Study_ID[MAX_STRING_LENGTH];
    char Modality[MAX_STRING_LENGTH];
    char SOP[MAX_STRING_LENGTH];
    int active;		/* if window is created */
    int AssociationLevel;	/* for sheduling round robin */
    int AssociationID;
    int window_x;	/* default locations */
    int window_y;
    Window window;
    XImage *xi;

}   Images_M;

static Images_M images[4];	/* 4=IMAGES variable space for displaying the
				 * transfered pictures */

/*----------------------------------------------------------------------*/

/* some forward declarations */
void CTNDISP_ConvertImageData(Image_Modul timage1, Images_M * timages, QUEUE_Element element);
void Image_Put_In_List(Images_M timages);
void DestroyImage(Images_M * images);
void CTNDISP_DisplayImageDataS(Images_M * images);

/*----------------------------------------------------------------------*/

void
ShowElement(QUEUE_Element tempelement)
{

    /*
     * the default are all Operations like Print,Find etc. I use the original
     * OperationState variable for initializising and destroying the
     * association handle. See dispuser.c for details. 
     */
#ifdef DEBUG
    printf("ctndisp recieved a element %s\n", Q_String[tempelement.Q_DSU_Operation]);
#endif

    switch (tempelement.Q_DSU_Operation) {

    case StartOperation:
	PutInList(tempelement);	/* Searches for a free place in the Showlist */
	ShowOperation(tempelement);
	break;
    case StopOperation:
	ShowOperation(tempelement);
	RemoveFromList(tempelement);
	/*
	 * Image_Remove_From_List(tempelement); 
	 */
	/* image is still displayed after stop association */
	break;
    case KillOperation:
	KillRemoveFromList(tempelement);

	break;
    default:
	ShowOperation(tempelement);

	break;
    }

    return;
    /* end ShowElement */
}

/********************************************************************/

void
PutInList(QUEUE_Element tempelement)
{				/* Puts a new association in the list */

    int counter;


    counter = 0;
    while ((counter < MAXWINDOWS) && (show_list[counter].element.Q_AssociationID != FREE)) {
	counter++;
    }


    if (counter == MAXWINDOWS) {
	fprintf(stderr, "CTNDISP: No free window to show association: %s\n", tempelement.Q_peerAE);
    } else {

	show_list[counter].element = tempelement;
	show_list[counter].imagecounter = 0;
#ifdef DEBUG
	printf("PutInList counter %d\n", counter);
#endif
    }
    return;
}

/********************************************************************/

void
RemoveFromList(QUEUE_Element tempelement)
{				/* Removes a  association from the list */

    int counter,
        flag;

    flag = 1;
    counter = 0;
    while ((counter < MAXWINDOWS) && (flag)) {
	if ((show_list[counter].element.Q_AssociationID == tempelement.Q_AssociationID) && (show_list[counter].element.Q_DSU_AssociationLevel
		== tempelement.Q_DSU_AssociationLevel)) {
	    flag = 0;
	} else {
	    counter++;
	}
    }

    if (counter == MAXWINDOWS) {
#ifdef DEBUG
	fprintf(stderr, "CTNDISP: Remove Association: %s failed. DSU_stopAssociation before DSU_startAssociation ?\n",
	    tempelement.Q_peerAE);
#endif
    } else {

	show_list[counter].element.Q_AssociationID = FREE;
    }
    return;
}

/********************************************************************/

void
KillRemoveFromList(QUEUE_Element tempelement)
{				/* Removes a  association from the list */

    int counter;

/* don`t worry about that, its another workaround */
    if (tempelement.Q_DSU_Percent == 0) {
	counter = 0;
	while (counter < MAXWINDOWS) {
	    show_list[counter].element.Q_AssociationID = FREE;
	    ShowStop(counter);
	    CTNDISP_RefreshWindowArea(counter);	/* update window */

	    counter++;
	}

    } else {
	counter = tempelement.Q_DSU_Percent - 1;
	if (counter < MAXWINDOWS) {
	    show_list[counter].element.Q_AssociationID = FREE;
	    ShowStop(counter);

	    CTNDISP_RefreshWindowArea(counter);	/* update window */

	}
    }
    return;
}

/********************************************************************/

void
ShowOperation(QUEUE_Element tempelement)
{

    int counter,
        flag;
    static int putflag = 0;

    counter = 0;
    flag = 1;
    while ((counter < MAXWINDOWS) && (flag)) {
	if ((show_list[counter].element.Q_AssociationID == tempelement.Q_AssociationID) && (show_list[counter].element.Q_DSU_AssociationLevel
		== tempelement.Q_DSU_AssociationLevel)) {
	    flag = 0;
	} else {
	    counter++;
	}
    }


    if ((counter == MAXWINDOWS) && !((MULTI) && (tempelement.Q_DSU_Operation == ImageOperation))) {
#ifdef DEBUG
	fprintf(stderr, "CTNDISP: Association: %s not in List.\n  DSU_startAssociation forgotten ?\n", tempelement.Q_peerAE);
#endif

    } else {
	if ((counter == MAXWINDOWS) && ((MULTI) && (tempelement.Q_DSU_Operation == ImageOperation))) {
	    tempelement.Q_DSU_Percent = 0;	/* else there will be garbage
						 * in show window */
	    tempelement.Q_DSU_AssociationLevel = NoAssociation;	/* else there will be
								 * garbage in show
								 * window */

	    PutInList(tempelement);
	    putflag = 1;	/* Searches for a free place in the Showlist */
	    flag = 1;
	    counter = 0;
	    while ((counter < MAXWINDOWS) && (flag)) {
		if ((show_list[counter].element.Q_AssociationID == tempelement.Q_AssociationID) && (show_list[counter].element.Q_DSU_AssociationLevel == tempelement.Q_DSU_AssociationLevel)) {
		    flag = 0;
		} else {
		    counter++;
		}
	    }
	}
    }
    if (counter == MAXWINDOWS) {
#ifdef DEBUG
	fprintf(stderr, "CTNDISP: Association: %s not in List.\n  DSU_startAssociation forgotten ?\n", tempelement.Q_peerAE);
#endif

    } else {
	show_list[counter].element = tempelement;


#ifdef DEBUG
	printf("counter %d\n", counter);
#endif
	switch (show_list[counter].element.Q_DSU_Operation) {
	case StartOperation:
	    ShowStart(counter);
	    break;
	case StoreOperation:
	    ShowStore(counter);
	    break;
	case FindOperation:
	    ShowFind(counter);
	    break;

	case MoveOperation:
	    ShowMove(counter);

	    break;

	case PrintOperation:
	    ShowPrint(counter);

	    break;

	case ImageOperation:
	    if (!MULTI) {
		ShowImage(counter);
	    } else {

		ShowImage(counter);
		if (putflag) {
		    RemoveFromList(tempelement);
		    putflag = 0;
		}
	    }



	    break;
	case StopOperation:
	    ShowStop(counter);
	    break;

	default:
	    break;
	}
    }
    CTNDISP_RefreshWindowArea(counter);	/* update window */
    return;
}

/*******************************************************************/
void 
ShowStart(int counter)
{
    int foo;

    /*
     * this is a little workaround to display images without association
     * shown 
     */
    if (show_list[counter].element.Q_DSU_AssociationLevel != NoAssociation) {

	sprintf(show_list[counter].show_strings[0], "%s \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
	sprintf(show_list[counter].show_strings[1], "%s: %s  --   %s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    show_list[counter].element.Q_myAE);

	for (foo = 2; foo < 20; foo++) {
	    strcpy(show_list[counter].show_strings[foo], "");
	}
    }
    return;
}

/*******************************************************************/
void 
ShowStore(int counter)
{
    int foo;

    if (show_list[counter].element.Q_DSU_OperationState != StopOperation) {
	sprintf(show_list[counter].show_strings[0], "%s   %s %d\n", Q_String[show_list[counter].element.Q_DSU_UserType],
	    Q_String[show_list[counter].element.Q_DSU_Operation], show_list[counter].imagecounter);
    } else {
	show_list[counter].imagecounter++;	/* stored a image */

	sprintf(show_list[counter].show_strings[0], "%s   \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
    }

    if (show_list[counter].element.Q_DSU_Direction == Incoming) {

	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    RIGHTDIRECTION, show_list[counter].element.Q_myAE);
    } else {
	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    LEFTDIRECTION, show_list[counter].element.Q_myAE);
    }

    for (foo = 2; foo < 20; foo++) {
	strcpy(show_list[counter].show_strings[foo], "");
    }

    return;
}

/*******************************************************************/
void 
ShowFind(int counter)
{
    int foo;

    if (show_list[counter].element.Q_DSU_OperationState != StopOperation) {
	sprintf(show_list[counter].show_strings[0], "%s   %s\n", Q_String[show_list[counter].element.Q_DSU_UserType],
	    Q_String[show_list[counter].element.Q_DSU_Operation]);
    } else {
	sprintf(show_list[counter].show_strings[0], "%s   \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
    }



    if (show_list[counter].element.Q_DSU_Direction == Incoming) {

	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    RIGHTDIRECTION, show_list[counter].element.Q_myAE);
    } else {
	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    LEFTDIRECTION, show_list[counter].element.Q_myAE);
    }

    for (foo = 2; foo < 20; foo++) {
	strcpy(show_list[counter].show_strings[foo], "");
    }


    return;
}

/*******************************************************************/
void 
ShowMove(int counter)
{
    int foo;

    if (show_list[counter].element.Q_DSU_OperationState != StopOperation) {
	sprintf(show_list[counter].show_strings[0], "%s   %s\n", Q_String[show_list[counter].element.Q_DSU_UserType],
	    Q_String[show_list[counter].element.Q_DSU_Operation]);
    } else {
	sprintf(show_list[counter].show_strings[0], "%s   \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
    }


    if (show_list[counter].element.Q_DSU_Direction == Incoming) {

	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    RIGHTDIRECTION, show_list[counter].element.Q_myAE);
    } else {
	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    LEFTDIRECTION, show_list[counter].element.Q_myAE);
    }

    for (foo = 2; foo < 20; foo++) {
	strcpy(show_list[counter].show_strings[foo], "");
    }

    return;
}

/*******************************************************************/
void 
ShowPrint(int counter)
{
    int foo;


    if (show_list[counter].element.Q_DSU_OperationState != StopOperation) {
	sprintf(show_list[counter].show_strings[0], "%s   %s\n", Q_String[show_list[counter].element.Q_DSU_UserType],
	    Q_String[show_list[counter].element.Q_DSU_Operation]);
    } else {
	sprintf(show_list[counter].show_strings[0], "%s   \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
    }



    if (show_list[counter].element.Q_DSU_Direction == Incoming) {

	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    RIGHTDIRECTION, show_list[counter].element.Q_myAE);
    } else {
	sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
	    LEFTDIRECTION, show_list[counter].element.Q_myAE);
    }
    for (foo = 2; foo < 20; foo++) {
	strcpy(show_list[counter].show_strings[foo], "");
    }

    return;
}

/*******************************************************************/
void 
ShowImage(int counter)
{
    int foo;
    static Images_M images_n;
    unsigned char *ptr;

    if (MULTI) {
	if (CalcStatus == INEXT) {
	    images_n.image = ishmaddr;

#ifdef DEBUG_FORK
	    printf("CalcStatus  INEXT\n");
#endif

	    if (show_list[counter].element.Q_DSU_OperationState != StopOperation) {
		sprintf(show_list[counter].show_strings[0], "%s   %s %d\n", Q_String[show_list[counter].element.Q_DSU_UserType],
		    Q_String[show_list[counter].element.Q_DSU_Operation], show_list[counter].imagecounter);

	    } else {
		sprintf(show_list[counter].show_strings[0], "%s   \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
	    }



	    if (show_list[counter].element.Q_DSU_Direction == Incoming) {

		sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
		    RIGHTDIRECTION, show_list[counter].element.Q_myAE);
	    } else {
		sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
		    LEFTDIRECTION, show_list[counter].element.Q_myAE);
	    }

	    for (foo = 2; foo < 20; foo++) {
		strcpy(show_list[counter].show_strings[foo], "");
	    }
	    /* if there is no association */
	    if (show_list[counter].element.Q_DSU_AssociationLevel == NoAssociation) {
		show_list[counter].element.Q_DSU_Percent = 0;

		for (foo = 0; foo < 20; foo++) {
		    strcpy(show_list[counter].show_strings[foo], "");
		}
	    }
	    tempimage.Pixel_Data_Length = 0;

	    GetImageData(show_list[counter].element.Q_ImageName, &tempimage, Rescaleflag, Windowflag, MULTI, CalcStatus);
#ifdef DEBUG_FORK
	    printf("Now fork %s \n", show_list[counter].element.Q_ImageName);
#endif
	    CalcStatus = ICALC;
	    switch (pid = fork()) {
	    case -1:
		printf("Fork error FATAL%d\n", pid);
		if ((shmctl(ishmID, IPC_RMID, shm_buf)) == -1) {
		    printf(": shmctl failed\n");

		}
		shared_error();	/* exits */

	    case 0:
		GetImageData(show_list[counter].element.Q_ImageName, &tempimage, Rescaleflag, Windowflag, MULTI, CalcStatus);

		if (tempimage.Pixel_Data_Length > 0)
		    CTNDISP_ConvertImageData(tempimage, &images_n, show_list[counter].element);
#ifdef DEBUG_FORK
		printf("sohn ende pid %d\n", pid);
#endif
		exit(42);
		return;

	    default:
		/* died = waitpid(pid, &status, WNOHANG|WUNTRACED); */
		/* died = wait(&status); */
		return;		/* father should work on, isn`t it always
				 * this way ? */


	    }
	} /* Calcstatus == INEXT */
	if (CalcStatus == IREADY) {
#ifdef DEBUG_FORK
	    printf("CalcStatus  IREADY\n");
#endif
	    /* images_n.image = ishmaddr; */

	    if (tempimage.Pixel_Data_Length > 0)
		CTNDISP_ConvertImageData(tempimage, &images_n, CalcElement);

	    if (tempimage.Pixel_Data_Length > 0)
		Image_Put_In_List(images_n);
	    /* calcstatus = IFREE set in ctndisp.c, main() */
	    return;
	} /* Calcstatus == IREADY */
	if ((CalcStatus == ICALC) || (CalcStatus == IFREE)) {
#ifdef DEBUG_FORK
	    printf("CalcStatus  IFREE or ICALC\n");
#endif

	    ImageQueue(&show_list[counter].element, PUT);	/* put image in queue */
	    /*
	     * this is the case if showelement is called from ctndisp.c
	     * direct after GetElement 
	     */

	    return;
	} /* Calcstatus == ICALC || IFREE */
	return;
    } else {			/*---------- not MULTI ---------------*/

	if ((ptr = (unsigned char *)calloc((unsigned)(IMAGESWIDTH * IMAGESHEIGHT + 2), sizeof(char))) == (unsigned char *)NULL)
	    malloc_error();
	images_n.image = ptr;

	if (show_list[counter].element.Q_DSU_OperationState != StopOperation) {
	    sprintf(show_list[counter].show_strings[0], "%s   %s %d\n", Q_String[show_list[counter].element.Q_DSU_UserType],
		Q_String[show_list[counter].element.Q_DSU_Operation], show_list[counter].imagecounter);

	} else {
	    sprintf(show_list[counter].show_strings[0], "%s   \n", Q_String[show_list[counter].element.Q_DSU_UserType]);
	}



	if (show_list[counter].element.Q_DSU_Direction == Incoming) {

	    sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
		RIGHTDIRECTION, show_list[counter].element.Q_myAE);
	} else {
	    sprintf(show_list[counter].show_strings[1], "%s: %s%s%s\n", show_list[counter].element.Q_peer, show_list[counter].element.Q_peerAE,
		LEFTDIRECTION, show_list[counter].element.Q_myAE);
	}

	for (foo = 2; foo < 20; foo++) {
	    strcpy(show_list[counter].show_strings[foo], "");
	}
	/* if there is no association */
	if (show_list[counter].element.Q_DSU_AssociationLevel == NoAssociation) {
	    for (foo = 0; foo < 20; foo++) {
		strcpy(show_list[counter].show_strings[foo], "");
	    }
	}
	tempimage.Pixel_Data_Length = 0;

	GetImageData(show_list[counter].element.Q_ImageName, &tempimage, Rescaleflag, Windowflag, MULTI, CalcStatus);
#ifdef DEBUG_IMAGE
	printf("%s \n", show_list[counter].element.Q_ImageName);
#endif
	if (tempimage.Pixel_Data_Length > 0) {

	    CTNDISP_ConvertImageData(tempimage, &images_n, show_list[counter].element);

	    Image_Put_In_List(images_n);
	}
	free(images_n.image);
	return;

    } /* if MULTI else branch */
}

/*******************************************************************/
void 
ShowStop(int counter)
{
    int foo;

    for (foo = 0; foo < 20; foo++) {
	strcpy(show_list[counter].show_strings[foo], "");
    }
    show_list[counter].element.Q_DSU_OperationState = StopOperation;

    return;
}

/*******************************************************************/
#define LOOKUP_ENTRIES 256
#define SHIFT_FACTOR 8
#define MIN_USEFUL_AMT	16
#define	WHITE 255

int 
CTNDISP_LoadGrayMap(int n)
{
    int i;
    XColor colors;
    unsigned long grayvalue;
    unsigned long graypixels[256];

    colors.flags = DoRed | DoGreen | DoBlue;

    for (i = 0; i < n; i++) {
	grayvalue = i * WHITE / (n - 1);
	colors.pixel = i;
	colors.red = colors.green = colors.blue =
	    grayvalue << SHIFT_FACTOR;
	if (!XAllocColor(G_display, G_cmap, &colors)) {
	    if (n <= MIN_USEFUL_AMT) {
		fprintf(stderr, "CTNDISP: Cannot get enough colors\n");
		return 0;
	    }
	    XFreeColors(G_display, G_cmap, graypixels, i, 0);
	    fprintf(stderr, "CTNDISP_LoadGrayMap: Recursive call to CTNDISP_LoadGrayMap [%d]\n", i);
	    return (CTNDISP_LoadGrayMap(i));
	} else {
#ifdef DEBUG_IMAGE
	    printf("Color %d: %d -> %d %d %d\n", i,
		colors.pixel, colors.red, colors.green, colors.blue);
#endif
	    graypixels[i] = colors.pixel;
	}
    }

    graylut[0] = graypixels[0];

#ifdef DEBUG_IMAGE
    printf(" Graylut:\n");
#endif

    for (i = 0; i < LOOKUP_ENTRIES; i++) {
	graylut[i] = graypixels[i / (LOOKUP_ENTRIES / n)];

#ifdef DEBUG_IMAGE
	printf(" %ld", graylut[i]);
#endif

    }
    graylutoffset = 0;		/* graylut [1]; */
/* it is the 1.3.1995. fixed the bug, with graylut. Now the image bytes are mapped through
graylut[] in routine Convert_Image_date. ...refuses tu die. FZ. */
#ifdef DEBUG_IMAGE
    printf(" \n");
#endif

    return (n);
}


/*******************************************************************/
int
CTNDISP_InitializeX(int ncolors)
{
    unsigned long valuemask;

    XSetWindowAttributes
	attributes;
    int width,
        height;

    /*
     * Connect the client to the X server...the default display is the
     * contents of the DISPLAY environment variable 
     */
    if ((G_display = XOpenDisplay(display_name)) == NULL) {
	fprintf(stderr, "CTNDISP: Cannot connect to X server\n");
	return 0;
    }
    G_screen = DefaultScreen(G_display);
    height = DisplayHeight(G_display, G_screen);
    width = DisplayWidth(G_display, G_screen);
    if ((G_display_height == -2) || (G_display_width == -2)) {
	if (height < width) {	/* to make displaywindow same side length */
	    G_display_height = (height - 30) / 2;	/* - 30 because of
							 * window title */
	    G_display_width = (width - 10) / 2;	/* G_display_height; */
	} else {

	    G_display_width = (width - 10) / 2;
	    G_display_height = (height - 30) / 2;	/* G_display_width; */
	}
    }
    if ((G_display_height == -3) || (G_display_width == -3)) {
	/*
	 * to make displaywindow same side length 
	 */
	G_display_height = (height - 30) / 2;	/* - 30 because of window
						 * title */

    }
    if ((G_display_height > height) || (G_display_height < 0))
	G_display_height = height;
    if ((G_display_width > width) || (G_display_width < 0))
	G_display_width = width;
    G_visual = DefaultVisual(G_display, G_screen);
    G_cmap = DefaultColormap(G_display, G_screen);


    /* create the graphics context */
    if (! CTNDISP_CreateGC()) {
	fprintf(stderr, "CTNDISP: Cannot get graphics context\n");
	return 0;
    }
    /*
     * Allocate a gray-scale colormap 
     */
    G_ncolors = CTNDISP_LoadGrayMap(ncolors);
#ifdef DEBUG_IMAGE
    printf(" G_ncolors: %d\n", G_ncolors);
#endif

    if (G_ncolors <= MIN_USEFUL_AMT) {
	fprintf(stderr, "CTNDISP_InitializeX: Insufficient gray levels [%d]\n", G_ncolors);
	return 0;
    }
    valuemask = CWBackPixel;
    attributes.background_pixel = graylut[G_ncolors / 2];
    G_big_window = XCreateWindow(G_display,
	RootWindow(G_display, G_screen),
	0, (height - G_display_height),	/* put window in the lower half */
	G_display_width,
	G_display_height,
	0, CopyFromParent,
	InputOutput,
	CopyFromParent,
	valuemask,
	&attributes);
    XStoreName(G_display, G_big_window, "European CTN Display Window");

    XSelectInput(G_display, G_big_window,
	ButtonReleaseMask | ExposureMask | StructureNotifyMask | SubstructureNotifyMask);
    /*
     * DEB
     * XSelectInput(G_display,RootWindow(G_display,G_screen),SubstructureNotif
     * yMask); 
     *
     * ButtonReleaseMask | SubstructureNotifyMask | ExposureMask |
     * StructureNotifyMask); 
     */

    XMapWindow(G_display, G_big_window);
    XFlush(G_display);
    return 1; /* success */
}


/*******************************************************************/
void
CTNDISP_RefreshImageData(XEvent event)
{
    int ctr,
        w;

    /*
     * if(images[0].window == event.xexpose.window ) printf("image 1 refresh
     * \n"); if(images[1].window == event.xexpose.window ) printf("image 2
     * refresh \n"); if(images[2].window == event.xexpose.window )
     * printf("image 3 refresh \n"); if(images[3].window ==
     * event.xexpose.window ) printf("image 4 refresh \n"); 
     */
    if (G_big_window == event.xexpose.window) {	/* printf("G_big_window
						 * refresh \n"); */
	return;
    }
    for (ctr = 0; ctr < IMAGES; ctr++) {
	if ((images[ctr].active == IMAGEACTIVE) && (images[ctr].window == event.xexpose.window)) {

#ifdef NSQCLEARIMAGE
	    XClearArea(G_display, images[ctr].window, 0, 0, IMAGESWIDTH, IMAGESHEIGHT + image_text_offset, False);
#endif
#ifdef NSQCENTERIMAGE
	    XPutImage(G_display, images[ctr].window, G_gclargeT, 
	        images[ctr].xi, 0, 0, 
		(IMAGESWIDTH / 2) - (images[ctr].xi->width / 2), 
		image_text_offset + ((IMAGESHEIGHT / 2) - (images[ctr].xi->height / 2)),
		(int)images[ctr].xi->width, (int)images[ctr].xi->height);
#else
	    XPutImage(G_display, images[ctr].window, G_gclargeT, images[ctr].xi, 0, 0, 0, image_text_offset,
		(int)images[ctr].xi->width, (int)images[ctr].xi->height);
#endif

	    w = XTextWidth(fontSMbold, images[ctr].AE_Titles, strlen(images[ctr].AE_Titles));
	    XDrawString(G_display, images[ctr].window, G_gcsmallTbold,
		1, image_text_height,
		images[ctr].AE_Titles, strlen(images[ctr].AE_Titles));



	    w = XTextWidth(fontSM, images[ctr].Patname, strlen(images[ctr].Patname));
	    XDrawString(G_display, images[ctr].window, G_gcsmallT,
		1, image_text_height * 2,
		images[ctr].Patname, strlen(images[ctr].Patname));
	    /*
	     * w = XTextWidth(fontSM, images[ctr].Study_ID,
	     * strlen(images[ctr].Study_ID)); XDrawString(G_display,
	     * images[ctr].window, G_gcsmallT, 1, image_text_height*3,
	     * images[ctr].Study_ID, strlen(images[ctr].Study_ID)); 
	     *
	     * w = XTextWidth(fontSM, images[ctr].Modality,
	     * strlen(images[ctr].Modality)); XDrawString(G_display,
	     * images[ctr].window, G_gcsmallT, 1, image_text_height*4,
	     * images[ctr].Modality, strlen(images[ctr].Modality)); 
	     */
	    w = XTextWidth(fontSM, images[ctr].SOP, strlen(images[ctr].SOP));
	    XDrawString(G_display, images[ctr].window, G_gcsmallT,
		1, image_text_height * 3,
		images[ctr].SOP, strlen(images[ctr].SOP));
	}
    }
    XFlush(G_display);
    return;

}

/*******************************************************************/
void
Image_Put_In_List(Images_M timages)
{
    static int roundrobin = -1;	/* cause i increment roungrobin first */

    int counter;

    /* first search if there is an inactiv window */

    counter = 0;
    while ((counter < IMAGES) && (images[counter].active == IMAGEACTIVE)) {
	counter++;
    }


    if (counter == IMAGES) {
	/* we have to destroy an older image */
	roundrobin++;
	if (roundrobin >= IMAGES)
	    roundrobin = 0;
	if (images[roundrobin].active == IMAGEACTIVE) {
#ifdef DEBUG_IMAGE
	    printf(" images[roundrobin].image : %ld\n", images[roundrobin].image);
#endif

	    DestroyImage(&images[roundrobin]);


	    images[roundrobin].active = IMAGEINACTIVE;
	    images[roundrobin].AssociationID = 0;

	    /* DestroyImage(&images[roundrobin]); */
	}
#ifdef NSQIMAGES
	images[roundrobin].nrows = timages.nrows;
	images[roundrobin].ncols = timages.ncols;
#endif
	strcpy(images[roundrobin].AE_Titles, timages.AE_Titles);
	strcpy(images[roundrobin].Study_ID, timages.Study_ID);
	strcpy(images[roundrobin].Modality, timages.Modality);
	strcpy(images[roundrobin].SOP, timages.SOP);
	strcpy(images[roundrobin].Patname, timages.Patname);
	/* put new in that space */
	images[roundrobin].active = IMAGEACTIVE;

	memcpy((char *)images[roundrobin].image, (char *)timages.image, IMAGESWIDTH * IMAGESHEIGHT * sizeof(char));

	images[roundrobin].AssociationLevel = timages.AssociationLevel;
	images[roundrobin].AssociationID = timages.AssociationID;

	CTNDISP_DisplayImageDataS(&images[roundrobin]);	/* and display it */

    } else {
	/* we found a free place */
	images[counter].active = IMAGEACTIVE;

#ifdef NSQIMAGES
	images[counter].nrows = timages.nrows;
	images[counter].ncols = timages.ncols;
#endif

	strcpy(images[counter].AE_Titles, timages.AE_Titles);
	strcpy(images[counter].Study_ID, timages.Study_ID);
	strcpy(images[counter].Modality, timages.Modality);
	strcpy(images[counter].SOP, timages.SOP);
	strcpy(images[counter].Patname, timages.Patname);

	memcpy((char *)images[counter].image, (char *)timages.image, IMAGESWIDTH * IMAGESHEIGHT * sizeof(char));

	images[counter].AssociationLevel = timages.AssociationLevel;
	images[counter].AssociationID = timages.AssociationID;


	CTNDISP_DisplayImageDataS(&images[counter]);	/* and display it */
    }


#ifdef DEBUG_IMAGE
    printf("IMAGE counter %d, roundrobin %d\n", counter, roundrobin);
#endif

    return;
}

/********************************************************************/

void
Image_Remove_From_List(QUEUE_Element tempelement)
{				/* Removes a  image from the list */
    /* uses global images array */

    int counter,
        flag;


    counter = 0;
    while (counter < IMAGES) {	/* maybe there are more images on one
				 * association */
	flag = 1;

	while ((counter < IMAGES) && (flag)) {
	    if ((images[counter].AssociationID == tempelement.Q_AssociationID) && (images[counter].AssociationLevel
		    == tempelement.Q_DSU_AssociationLevel)) {
		flag = 0;
	    } else {
		counter++;
	    }
	}

	if (counter != IMAGES) {/* found the image to smash (destroy) */
#ifdef DEBUG_IMAGE
	    printf("remove images[counter].image : %ld\n", images[counter].image);
#endif

	    DestroyImage(&images[counter]);


	    images[counter].active = IMAGEINACTIVE;
	    images[counter].AssociationID = 0;

#ifdef DEBUG_IMAGE
	    printf("Remove  counter %d\n", counter);
#endif
	}
    }

    return;

}

/*******************************************************************/
void 
CTNDISP_DisplayImageDataS(Images_M * images)
{
    unsigned long valuemask;

    XSetWindowAttributes
	attributes;

    char patient_string[256];
    int w;

#ifdef DEBUG_IMAGE
    printf("IMAGE x %d y %d \n", images->window_x, images->window_y);
#endif

    valuemask = 0;
    valuemask = CWBackPixel;
    attributes.background_pixel = graylut[G_ncolors - 1];

    images->window = XCreateWindow(G_display,
	G_big_window,
	images->window_x, images->window_y,
	IMAGESWIDTH, IMAGESHEIGHT + image_text_offset /* rows */ ,
	0, CopyFromParent,
	InputOutput,
	CopyFromParent,
	valuemask,
	&attributes);

    XMapWindow(G_display, images->window);

    XSelectInput(G_display, images->window, ExposureMask);


    images->xi = XCreateImage(G_display,
	G_visual,
	8,			/* or 16 */
	ZPixmap,
	0,
	(char *)images->image,
#ifdef NSQIMAGES
	images->ncols, images->nrows,
#else
	IMAGESWIDTH, IMAGESHEIGHT,	/* rows */
#endif
	8, 0);			/* 16,0 orig. ZPixmap */

#ifdef NSQIMAGES
    images->xi->width = images->ncols;
    images->xi->height = images->nrows;
#else
    images->xi->width = IMAGESWIDTH;
    images->xi->height = IMAGESHEIGHT;
#endif
    images->xi->xoffset = 0;
    images->xi->format = ZPixmap;
    images->xi->data = (char *)images->image;
    images->xi->byte_order = MSBFirst;
    images->xi->bits_per_pixel = 8;	/* 16 */
    images->xi->depth = 8;	/* 8 !! */
    images->xi->bitmap_pad = 8;	/* 16 */
    images->xi->bitmap_unit = 8;/* 16 */

#ifdef NSQCLEARIMAGE
    XClearArea(G_display, images->window, 0, 0, IMAGESWIDTH, IMAGESHEIGHT + image_text_offset, False);
#endif

#ifdef NSQCENTERIMAGE
    XPutImage(G_display, images->window, G_gclargeT, 
	      images->xi, 0, 0, 
	      (IMAGESWIDTH / 2) - (images->xi->width / 2), 
	      image_text_offset + ((IMAGESHEIGHT / 2) - (images->xi->height / 2)),
	      (int)images->xi->width, (int)images->xi->height);
#else
    XPutImage(G_display, images->window, G_gclargeT,
	images->xi, 0, 0, 0, image_text_offset, (int)images->xi->width, (int)images->xi->height);
#endif

    w = XTextWidth(fontSMbold, images->AE_Titles, strlen(images->AE_Titles));
    XDrawString(G_display, images->window, G_gcsmallTbold,
	1, image_text_height,
	images->AE_Titles, strlen(images->AE_Titles));
    sprintf(patient_string, "%s %s %s", images->Modality, images->Study_ID, images->Patname);
    strcpy((images->Patname), patient_string);
    w = XTextWidth(fontSM, images->Patname, strlen(images->Patname));
    XDrawString(G_display, images->window, G_gcsmallT,
	1, image_text_height * 2,
	images->Patname, strlen(images->Patname));


    /*
     * w = XTextWidth( fontSM, images->Study_ID, strlen(images->Study_ID) );
     * XDrawString (G_display, images->window, G_gcsmallT, 1,
     * image_text_height *3, images->Study_ID, strlen (images->Study_ID)); 
     *
     * w = XTextWidth( fontSM, images->Modality, strlen(images->Modality) );
     * XDrawString (G_display, images->window, G_gcsmallT, 1,
     * image_text_height *4, images->Modality, strlen (images->Modality)); 
     *
     */

    w = XTextWidth(fontSM, images->SOP, strlen(images->SOP));
    XDrawString(G_display, images->window, G_gcsmallT,
	1, image_text_height * 3,
	images->SOP, strlen(images->SOP));


    XFlush(G_display);
    /* printf("%s\n",(images->Patname)); */
    return;
}

#ifdef NSQIMAGES
void
calc_displayed_image_size(float cols, float rows, int *ncols, int *nrows)
{
    float avail_cols;
    float avail_rows;
    float ratio_cols;
    float ratio_rows;
    float the_ratio;

    avail_cols = IMAGESWIDTH;
    avail_rows = IMAGESHEIGHT;
    
    ratio_cols = cols / avail_cols;
    ratio_rows = rows / avail_rows;
    
    if (ratio_rows > ratio_cols) {
        the_ratio = ratio_rows;
    } else {
        the_ratio = ratio_cols;
    }
    
    *ncols = (int)(cols / the_ratio);
    *nrows = (int)(rows / the_ratio);
}


#endif

/*******************************************************************/
void
CTNDISP_ConvertImageData(Image_Modul timage1, Images_M * timages, QUEUE_Element element)
{
    char tempstring[256];
    int xctr,
        yctr,
        ctr;
    unsigned short maxvalue,
        minvalue,
        shiftfactor,
        palettemaxvalue,
        paletteminvalue,
        sfoo,
        offset,
        Window_max,
        Window_min;
    short foo;
    unsigned long bytefactorl,
        hibitmask,
        bytemask;
    unsigned short *graydata,
       *graydatahelp,
        grayvalue,
       *redptr,
       *greenptr,
       *blueptr,
       *graylookup;
    unsigned char *cptr,
       *cptrconvhelp,
       *credptr,
       *cgreenptr,
       *cblueptr;

    float scalex,
        scaley,
        xctri,
        yctri,
        bytefactor,
        palette_to_gray_factor;
    unsigned short *ptr,
       *ptrconvhelp;
    unsigned char *ptrconv;

    int image_display_width;
    int image_display_height;

    image_display_width = IMAGESWIDTH;
    image_display_height = IMAGESHEIGHT;
    
#ifdef NSQIMAGES
    /* retain aspect ratio of image when scaling down */
    calc_displayed_image_size(timage1.Columns, timage1.Rows, 
	&timages->ncols, &timages->nrows);

    image_display_width = timages->ncols;
    image_display_height = timages->nrows;

    scalex = (float)timage1.Columns / (float)image_display_width;
    scaley = (float)timage1.Rows / (float)image_display_height;

#ifdef DEBUG_IMAGE
    printf("++++NSQIMAGES++++\n");
    printf("IMAGESWIDTH=%d\n", IMAGESWIDTH);
    printf("IMAGESHEIGHT=%d\n", IMAGESHEIGHT);
    printf("image_display_width=%d\n", image_display_width);
    printf("image_display_height=%d\n", image_display_height);
    printf("scalex=%f\n", scalex);
    printf("scaley=%f\n", scaley);
    printf("----NSQIMAGES----\n");
#endif
#endif


    if ((!MULTI) || ((MULTI) && (CalcStatus == ICALC))) {
	if ((strncmp(timage1.Photometric_Interpretation, "MONOCHROME1", 
            strlen("MONOCHROME1")) == 0) || 
            (strncmp(timage1.Photometric_Interpretation, "MONOCHROME2", 
            strlen("MONOCHROME2")) == 0) || 
            (strlen(timage1.Photometric_Interpretation) == 0)) {
#ifdef DEBUG_IMAGE
	    printf("MONOCHROME  CALC : %s \n", timage1.Photometric_Interpretation);
#endif

	    /* scale image, put image in timages, */
	    scalex = (float)timage1.Columns / (float)image_display_width;
	    scaley = (float)timage1.Rows / (float)image_display_height;

	    shiftfactor = (timage1.High_Bit + 1) - timage1.Bits_Stored;
	    /*
	     * if stored == 0111 1110,highbit==6,bitsstored == 6,then
	     * shiftfactor =1, data >> 1 !! 
	     */


	    bytefactorl = 0;
	    hibitmask = 1L;	/* for masking the highbit to calc 2
				 * Komplement */

	    for (ctr = 0; ctr < (int)timage1.Bits_Stored; ctr++) {
		bytefactorl <<= 1;
		bytefactorl |= 1;
		hibitmask <<= 1;

	    }
	    hibitmask >>= 1;	/* if there are 2 bits stored, then hibit
				 * mask == (long)0000 0010 */
	    bytemask = bytefactorl >> 1;

#ifdef DEBUG_IMAGE
	    printf("bytefactorl : %ld\n", bytefactorl);
#endif

	    Window_min = 0;
	    Window_max = 0;


	    /* **** here we differ betwwen 8 and 16 bit images */

	    if (timage1.Bits_Stored > 8) {

		ptr = (unsigned short *)timage1.Pixel_Data;

		maxvalue = 0;
		minvalue = 65535;

		if (timage1.Pixel_Representation == 0 /* 0!!!!!!!!! */ ) {
#ifdef DEBUG_IMAGE
		    printf("Unsigned value 16 bit mark 1 bytemask,%lx hibitmask %lx  bytefactorl %lx\n", bytemask,
			hibitmask, bytefactorl);

#endif
		    offset = 0;
		    if (timage1.Window_Width > 0.0) {
			if (timage1.Window_Center < 0.0) {
			    timage1.Window_Center += (float)hibitmask;
			}
			foo = (short)(timage1.Window_Center - (timage1.Window_Width / 2));
			if (foo < 0) {
			    offset = (unsigned short)abs(foo);
			    foo += (short)offset;
			};
#ifdef DEBUG_IMAGE
			printf("foo %d\n", foo);
#endif

			Window_min = (unsigned short)foo;

			foo = (short)(timage1.Window_Center + (timage1.Window_Width / 2));
			Window_max = (unsigned short)(foo + (short)offset);

#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u\n", Window_min, Window_max);
#endif
			/*
			 * Window_min = ((Window_min & hibitmask) == 0L) ?
			 * ((Window_min & bytemask) + hibitmask) :
			 * (Window_min & bytemask); Window_max = ((Window_max
			 * & hibitmask) == 0L) ? ((Window_max & bytemask) +
			 * hibitmask) : (Window_max & bytemask); 
			 */
			if (Window_min > Window_max) {
			    sfoo = Window_min;
			    Window_min = Window_max;
			    Window_max = sfoo;
			}
#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u\n", Window_min, Window_max);
#endif

		    }
#ifdef DEBUG_IMAGE
		    printf(" timage1.Rescale_Intercept%f timage1.Rescale_Slope%f timage1.Window_Center %f timage1.Window_Width%f\n",
			timage1.Rescale_Intercept, timage1.Rescale_Slope, timage1.Window_Center, timage1.Window_Width);


#endif

		    for (ctr = 0; ctr < (int)(timage1.Rows * timage1.Columns); ctr++) {


			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			/**ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);*/

			*ptr = (unsigned short)MAX(((timage1.Rescale_Slope * (float)*ptr + timage1.Rescale_Intercept) + (float)offset), 0);
			*ptr = (unsigned short)(((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr));

			if (maxvalue < *ptr)
			    maxvalue = *ptr;
			if (minvalue > *ptr) {
			    minvalue = *ptr;
			}
			ptr++;
		    }
		} else {
#ifdef DEBUG_IMAGE
		    printf("Two Komplement value \n");
		    printf(" bytemask,%lx hibitmask %lx  bytefactorl %lx\n", bytemask, hibitmask, bytefactorl);

#endif
		    /* scaling factors racalc */
		    offset = 0;

		    if (timage1.Window_Width > 0.0) {

			foo = (short)(timage1.Window_Center - (timage1.Window_Width / 2));
			if (foo < 0) {
			    offset = (unsigned short)abs(foo);
			    foo += (short)offset;
			};
#ifdef DEBUG_IMAGE
			printf("foo %d\n", foo);
#endif
			Window_min = (unsigned short)foo;

			foo = (short)(timage1.Window_Center + (timage1.Window_Width / 2));
			Window_max = (unsigned short)(foo + (short)offset);
#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u offset %u\n", Window_min, Window_max, offset);
#endif


			Window_min = ((Window_min & hibitmask) == 0L) ? ((Window_min & bytemask) + hibitmask) :
			    (Window_min & bytemask);
			Window_max = ((Window_max & hibitmask) == 0L) ? ((Window_max & bytemask) + hibitmask) :
			    (Window_max & bytemask);

#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u\n", Window_min, Window_max);
#endif


		    }
#ifdef DEBUG_IMAGE
		    printf(" timage1.Rescale_Intercept%f timage1.Rescale_Slope%f timage1.Window_Center %f timage1.Window_Width%f\n",
			timage1.Rescale_Intercept, timage1.Rescale_Slope, timage1.Window_Center, timage1.Window_Width);

#endif

		    for (ctr = 0; ctr < (int)(timage1.Rows * timage1.Columns); ctr++) {

			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);

			*ptr = (unsigned short)MAX((timage1.Rescale_Slope * (float)*ptr + timage1.Rescale_Intercept + (float)offset), 0);
			*ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);



			if (maxvalue < *ptr) {
			    maxvalue = *ptr;
#ifdef DEBUG_IMAGE

			    if (ctr < 50)
				printf("signed value mark 1 %u ,%lx (*ptr & hibitmask) %lx  (*ptr & bytemask) %lx\n",
				    *ptr, (*ptr & bytemask) + hibitmask, (*ptr & hibitmask), (*ptr & bytemask));
#endif


			}
			if (minvalue > *ptr) {
			    minvalue = *ptr;
#ifdef DEBUG_IMAGE
			    if (ctr < 50)
				printf("signed value mark 2 %u ,%lx (*ptr & hibitmask) %lx  (*ptr & bytemask) %lx\n",
				    *ptr, (*ptr & bytemask) + hibitmask, (*ptr & hibitmask), (*ptr & bytemask));
#endif


			}
			ptr++;
		    }

		}

#ifdef DEBUG_IMAGE
		printf("maxvalue %u,minvalue %u,Window_min %u Window_max %u\n", maxvalue, minvalue, Window_min, Window_max);
#endif
		if ((Window_min > (minvalue)) && (Window_min < (maxvalue)))
		    minvalue = Window_min;
		if ((Window_max < (maxvalue)) && (Window_max > minvalue))
		    maxvalue = Window_max;


#ifdef DEBUG_IMAGE
		printf("maxvalue %d,minvalue %d\n", maxvalue, minvalue);
#endif

		bytefactor = (((float)(maxvalue - minvalue)) / ((float)(G_ncolors) - (float)graylutoffset));
		if (bytefactor == 0.0)
		    bytefactor = 1.0;


		yctri = 0.0;
		ptr = (unsigned short *)timage1.Pixel_Data;
#ifdef DEBUG_IMAGE
		printf("scalex : %f\n", scalex);
		printf("scaley : %f\n", scaley);
		printf("shiftfactor : %d\n", shiftfactor);
		printf("bytefactor : %f\n", bytefactor);
		printf("maxvalue : %d\n", maxvalue);
		printf("minvalue : %d\n", minvalue);
#endif

		ptrconv = (unsigned char *)timages->image;

		ptrconvhelp = ptr;
		if (strncmp(timage1.Photometric_Interpretation, "MONOCHROME1",
                    strlen("MONOCHROME1")) == 0) {
		    for (yctr = 0; yctr < image_display_height; yctr++) {
			xctri = 0.0;

			for (xctr = 0; xctr < image_display_width; xctr++) {
			    ptrconvhelp = ptr + ((int)xctri + ((int)yctri) * timage1.Columns);

			    if (*ptrconvhelp < minvalue)
				*ptrconvhelp = minvalue;
			    if (*ptrconvhelp > maxvalue)
				*ptrconvhelp = maxvalue;

			    *ptrconv = (unsigned char)graylut[((G_ncolors) - ((unsigned short)((float)(*ptrconvhelp -
						minvalue) / (float)bytefactor)))];
#ifdef DEBUG_IMAGE
			    if ((*ptrconv < graylutoffset) || (*ptrconv > (G_ncolors)))
				printf("*ptrconv: %d  \n", (int)*ptrconv);
#endif
			    ptrconv++;

			    xctri += scalex;


			}
			yctri += scaley;
			/*
			 * (printf("yctri : %d %f \n",(int)yctri,yctri); 
			 */
		    }

		} else {	/* monochrom 2 and default */

		    for (yctr = 0; yctr < image_display_height; yctr++) {
			xctri = 0.0;

			for (xctr = 0; xctr < image_display_width; xctr++) {
			    ptrconvhelp = ptr + ((int)xctri + ((int)yctri) * timage1.Columns);
			    /**ptrconvhelp=MIN(MAX(*ptrconvhelp,minvalue),maxvalue);*/
			    if (*ptrconvhelp < minvalue)
				*ptrconvhelp = minvalue;
			    if (*ptrconvhelp > maxvalue)
				*ptrconvhelp = maxvalue;


			    *ptrconv = (unsigned char)graylut[(graylutoffset + ((unsigned short)((float)(*ptrconvhelp
						- minvalue) / (float)bytefactor)))];


			    ptrconv++;

			    xctri += scalex;


			}
			yctri += scaley;
			/*
			 * (printf("yctri : %d %f \n",(int)yctri,yctri); 
			 */
		    }

		} /* differ between mono 1 and 2 */



	    }
	    /* 16 bit */
	    else {
		/* *******************its a 8 bit picture */
		cptr = (unsigned char *)timage1.Pixel_Data;


		maxvalue = 0;
		minvalue = 65535;
		if (timage1.Pixel_Representation == 0) {
#ifdef DEBUG_IMAGE
		    printf("Unsigned value 8 bit mark 1 bytemask,%lx hibitmask %lx  bytefactorl %lx\n", bytemask,
			hibitmask, bytefactorl);
#endif
		    offset = 0;

		    if (timage1.Window_Width > 0.0) {
			if (timage1.Window_Center < 0.0) {
			    timage1.Window_Center += (float)hibitmask;
			}
			foo = (short)(timage1.Window_Center - (timage1.Window_Width / 2));
			if (foo < 0) {
			    offset = (unsigned short)abs(foo);
			    foo += (short)offset;
			};
#ifdef DEBUG_IMAGE
			printf("foo %d\n", foo);
#endif

			Window_min = (unsigned short)foo;

			foo = (short)(timage1.Window_Center + (timage1.Window_Width / 2));
			Window_max = (unsigned short)(foo + (short)offset);

#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u\n", Window_min, Window_max);
#endif
			/*
			 * Window_min = ((Window_min & hibitmask) == 0L) ?
			 * ((Window_min & bytemask) + hibitmask) :
			 * (Window_min & bytemask); Window_max = ((Window_max
			 * & hibitmask) == 0L) ? ((Window_max & bytemask) +
			 * hibitmask) : (Window_max & bytemask); 
			 */
#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u\n", Window_min, Window_max);
#endif


		    }
#ifdef DEBUG_IMAGE
		    printf(" timage1.Rescale_Intercept%f timage1.Rescale_Slope%f timage1.Window_Center %f timage1.Window_Width%f\n",
			timage1.Rescale_Intercept, timage1.Rescale_Slope, timage1.Window_Center, timage1.Window_Width);


#endif


		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
			*cptr = ((((*cptr)) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = (unsigned char)MAX(((timage1.Rescale_Slope * (float)*cptr + timage1.Rescale_Intercept) + (float)offset), 0);
			/**cptr = ((*cptr & hibitmask) == 0L) ? ((*cptr & bytemask) + hibitmask) : (*cptr & bytemask);*/
			*cptr = (unsigned char)(((*cptr & 0x80) == 0x80) ? (*cptr - 0x100) : (*cptr));


			if (maxvalue < *cptr)
			    maxvalue = *cptr;
			if (minvalue > *cptr)
			    minvalue = *cptr;
			cptr++;
		    }
		} else {	/* its two complement */


		    offset = 0;

		    if (timage1.Window_Width > 0.0) {

			foo = (short)(timage1.Window_Center - (timage1.Window_Width / 2));
			if (foo < 0) {
			    offset = (unsigned short)abs(foo);
			    foo += (short)offset;
			};
#ifdef DEBUG_IMAGE
			printf("foo %d\n", foo);
#endif
			Window_min = (unsigned short)foo;

			foo = (short)(timage1.Window_Center + (timage1.Window_Width / 2));
			Window_max = (unsigned short)(foo + (short)offset);
#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u offset %u\n", Window_min, Window_max, offset);
#endif


			Window_min = ((Window_min & hibitmask) == 0L) ? ((Window_min & bytemask) + hibitmask) :
			    (Window_min & bytemask);
			Window_max = ((Window_max & hibitmask) == 0L) ? ((Window_max & bytemask) + hibitmask) :
			    (Window_max & bytemask);

#ifdef DEBUG_IMAGE
			printf("Window_min %u Window_max %u\n", Window_min, Window_max);
#endif

		    }
#ifdef DEBUG_IMAGE
		    printf(" timage1.Rescale_Intercept%f timage1.Rescale_Slope%f timage1.Window_Center %f timage1.Window_Width%f\n",
			timage1.Rescale_Intercept, timage1.Rescale_Slope, timage1.Window_Center, timage1.Window_Width);

#endif
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
			*cptr = (((*cptr) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = (unsigned char)MAX((timage1.Rescale_Slope * (float)*cptr + timage1.Rescale_Intercept + (float)offset), 0);
			*cptr = (unsigned char)(((*cptr & hibitmask) == 0L) ? ((*cptr & bytemask) + hibitmask) : (*cptr & bytemask));

			if (maxvalue < *cptr) {

			    maxvalue = *cptr;
			}
			if (minvalue > *cptr)
			    minvalue = *cptr;
			cptr++;
		    }
		}
#ifdef DEBUG_IMAGE
		printf("maxvalue %u,minvalue %u,Window_min %u Window_max %u\n", maxvalue, minvalue, Window_min, Window_max);
#endif
		if ((Window_min > (minvalue)) && (Window_min < (maxvalue)))
		    minvalue = Window_min;
		if ((Window_max < (maxvalue)) && (Window_max > minvalue))
		    maxvalue = Window_max;


#ifdef DEBUG_IMAGE
		printf("maxvalue %d,minvalue %d\n", maxvalue, minvalue);
#endif

		bytefactor = (((float)(maxvalue - minvalue)) / ((float)(G_ncolors) - (float)graylutoffset));
		if (bytefactor == 0.0)
		    bytefactor = 1.0;


		yctri = 0.0;
		cptr = (unsigned char *)timage1.Pixel_Data;
#ifdef DEBUG_IMAGE
		printf("scalex : %f\n", scalex);
		printf("scaley : %f\n", scaley);
		printf("shiftfactor : %d\n", shiftfactor);
		printf("bytefactor : %f\n", bytefactor);
		printf("maxvalue : %d\n", maxvalue);
		printf("minvalue : %d\n", minvalue);
#endif


		ptrconv = (unsigned char *)timages->image;	/* destination of
								 * convert */
		cptrconvhelp = cptr;	/* source of convert */
		if (strncmp(timage1.Photometric_Interpretation, "MONOCHROME1",
                  strlen("MONOCHROME1")) == 0) {
		    for (yctr = 0; yctr < image_display_height; yctr++) {
			xctri = 0.0;

			for (xctr = 0; xctr < image_display_width; xctr++) {
			    cptrconvhelp = cptr + ((int)xctri + ((int)yctri) * timage1.Columns);

			    if (*cptrconvhelp < minvalue)
				*cptrconvhelp = (unsigned char)minvalue;
			    if (*cptrconvhelp > maxvalue)
				*cptrconvhelp = (unsigned char)maxvalue;
			    *ptrconv = (unsigned short)graylut[((G_ncolors) - ((unsigned char)((float)(*cptrconvhelp -
						minvalue) / (float)bytefactor)))];


#ifdef DEBUG_IMAGE
			    if ((*ptrconv < graylutoffset) || (*ptrconv > (G_ncolors)))
				printf("*ptrconv: %d  \n", (int)*ptrconv);
#endif

			    ptrconv++;

			    xctri += scalex;


			}
			yctri += scaley;
		    }
		} else {	/* monochrom 2 or default */
		    for (yctr = 0; yctr < image_display_height; yctr++) {
			xctri = 0.0;

			for (xctr = 0; xctr < image_display_width; xctr++) {
			    cptrconvhelp = cptr + ((int)xctri + ((int)yctri) * timage1.Columns);
			    if (*cptrconvhelp < minvalue)
				*cptrconvhelp = (unsigned char)minvalue;
			    if (*cptrconvhelp > maxvalue)
				*cptrconvhelp = (unsigned char)maxvalue;

			    *ptrconv = (unsigned short)graylut[(graylutoffset + ((unsigned short)((float)(*cptrconvhelp - (unsigned char)minvalue) / (float)bytefactor)))];

			    ptrconv++;

			    xctri += scalex;


			}
			yctri += scaley;
			/*
			 * (printf("yctri : %d %f \n",(int)yctri,yctri); 
			 */
		    }

		} /* differ between mono 1 and 2 */




	    } /* 8 bit picture */
	} else if (strncmp(timage1.Photometric_Interpretation, "RGB", 
          strlen("RGB")) == 0) {

	    /*********************** RGB ***********************/
#ifdef DEBUG_IMAGE
	    printf("RGB PICTURE \n");
#endif
	    /*
	     * ptrconv=(unsigned short*)timages->image; for (yctr = 0; yctr <
	     * image_display_height; yctr++) { 
	     *
	     * for (xctr = 0; xctr < image_display_width; xctr++) { 
	     *
	     * ptrconv=(unsigned short) (xctr > (G_ncolors)) ? (G_ncolors):
	     * xctr; ptrconv++; 
	     *
	     * } } 
	     *
	     */
	    if ((graydata = (unsigned short *)calloc((unsigned long)(((unsigned long)timage1.Rows * (unsigned long)timage1.Columns
				+ 2) * sizeof(unsigned short)), sizeof(unsigned char))) == (unsigned short *)NULL)
		malloc_error();
#ifdef DEBUG_IMAGE
	    printf("graydataarray size %ld \n", sizeof(graydata));
	    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr += 10000) {
		printf("%ld \n", ctr);
		graydatahelp = graydata + (ctr);
		*graydatahelp = 17;
	    }

#endif

	    /* scale image, put image in timages, */
	    scalex = (float)timage1.Columns / (float)image_display_width;
	    scaley = (float)timage1.Rows / (float)image_display_height;

	    shiftfactor = (timage1.High_Bit + 1) - timage1.Bits_Stored;
	    /*
	     * if stored == 0111 1110,highbit==6,bitsstored == 6,then
	     * shiftfactor =1, data >> 1 !! 
	     */


	    bytefactorl = 0;
	    hibitmask = 1L;	/* for masking the highbit to calc 2
				 * Komplement */

	    for (ctr = 0; ctr < timage1.Bits_Stored; ctr++) {
		bytefactorl <<= 1;
		bytefactorl |= 1;
		hibitmask <<= 1;

	    }
	    hibitmask >>= 1;	/* if there are 2 bits stored, then hibit
				 * mask == (long)0000 0010 */
	    bytemask = bytefactorl >> 1;

#ifdef DEBUG_IMAGE
	    printf("bytefactorl : %ld\n", bytefactorl);
#endif



	    if (timage1.Bits_Stored > 8) {
		ptr = (unsigned short *)timage1.Pixel_Data;

		maxvalue = 0;
		minvalue = 0xFFFF;

		if (timage1.Pixel_Representation == 0) {
#ifdef DEBUG_IMAGE
		    printf("Unsigned value \n");
		    printf("signed value mark 1 bytemask,%lx hibitmask %lx  bytefactorl %lx\n", bytemask, hibitmask,
			bytefactorl);

#endif


		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {


			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);

			ptr++;
			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);

			ptr++;
			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);

			ptr++;
		    }
		} else {

		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {

			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
			ptr++;
			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
			ptr++;
			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
			ptr++;
		    }

		}

		/* now conver the rgb values in grayvalues */

		ptr = (unsigned short *)timage1.Pixel_Data;
		ptrconvhelp = ptr;

		graydatahelp = graydata;	/* destination of rgb to gray
						 * converted array */
		redptr = ptr;	/* ptr if planarconfiguration 1
				 * (R1R2R3...G1G2G3...B1B2B3 */
		greenptr = redptr + (timage1.Rows * timage1.Columns);
		blueptr = greenptr + (timage1.Rows * timage1.Columns);

		maxvalue = 0;
		minvalue = 0xFFFF;

		if (timage1.Planar_Configuration == 0) {
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
			/*
			 * I multiply the RGB values with a factor near the
			 * normal used factors for speed reasons. with  8 bit
			 * data it is possible, cause short is 2 byte long. 
			 */
			grayvalue = (unsigned short)*ptrconvhelp++ * 2;	/** 0.3  red */
			grayvalue += (unsigned short)*ptrconvhelp++ * 3;	/* * 0.59  green */
			grayvalue += (unsigned short)*ptrconvhelp++;	/** 0.11 blue */

			*graydatahelp = (unsigned short)grayvalue;	/* store the grayvalues */
			graydatahelp++;
			if (maxvalue < grayvalue)
			    maxvalue = grayvalue;
			if (minvalue > grayvalue)
			    minvalue = grayvalue;
		    }
		} else {

		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
			grayvalue = (unsigned short)*redptr++ * 2;	/** 0.3 red */
			grayvalue += (unsigned short)*greenptr++ * 3;	/* * 0.59 green */
			grayvalue += (unsigned short)*blueptr++;	/** 0.11 blue */

			*graydatahelp = (unsigned short)grayvalue;	/* store the grayvalues */
			graydatahelp++;
			if (maxvalue < grayvalue)
			    maxvalue = grayvalue;
			if (minvalue > grayvalue)
			    minvalue = grayvalue;
		    }


		}


	    }
	    /* 16 bit */
	    else {
		/* *******************its a 8 bit picture */


#ifdef DEBUG_IMAGE
		cptr = (unsigned char *)timage1.Pixel_Data;

		/*
		 * for (ctr = 0; ctr < 255; ctr++) { cptr++ = (unsigned char)
		 * ctr; cptr++ = (unsigned char) ctr; cptr++ = (unsigned
		 * char) ctr; } 
		 */
#endif

		cptr = (unsigned char *)timage1.Pixel_Data;

		if (timage1.Pixel_Representation == 0) {
#ifdef DEBUG_IMAGE
		    printf("Unsigned value timage1.Rows * timage1.Columns : %d \n", timage1.Rows * timage1.Columns);
#endif
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {

			*cptr = ((((*cptr)) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & 0x80) == 0x80) ? (*cptr - 0x100) : (*cptr);

			cptr++;
			*cptr = ((((*cptr)) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & 0x80) == 0x80) ? (*cptr - 0x100) : (*cptr);

			cptr++;
			*cptr = ((((*cptr)) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & 0x80) == 0x80) ? (*cptr - 0x100) : (*cptr);

			cptr++;
		    }

		} else {	/* 2 komplement */
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {

			*cptr = (((*cptr) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & hibitmask) == 0L) ? ((*cptr & bytemask) + hibitmask) : (*cptr &
			    bytemask);
			cptr++;
			*cptr = (((*cptr) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & hibitmask) == 0L) ? ((*cptr & bytemask) + hibitmask) : (*cptr &
			    bytemask);
			cptr++;
			*cptr = (((*cptr) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & hibitmask) == 0L) ? ((*cptr & bytemask) + hibitmask) : (*cptr &
			    bytemask);
			cptr++;
		    }
		}


		/* now conver the rgb values in grayvalues */

		cptr = (unsigned char *)timage1.Pixel_Data;
		cptrconvhelp = cptr;

		graydatahelp = graydata;	/* destination of rgb to gray
						 * converted array */
		credptr = cptr;	/* ptr if planarconfiguration 1
				 * (R1R2R3...G1G2G3...B1B2B3 */
		cgreenptr = credptr + (timage1.Rows * timage1.Columns);
		cblueptr = cgreenptr + (timage1.Rows * timage1.Columns);

		maxvalue = 0;
		minvalue = 0xFFFF;

		if (timage1.Planar_Configuration == 0) {
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
			/*
			 * I multiply the RGB values with a factor near the
			 * normal used factors for speed reasons. with  8 bit
			 * data it is possible, cause short is 2 byte long. 
			 */
			grayvalue = (unsigned short)*cptrconvhelp++ * 2;	/** 0.3  red */
			grayvalue += (unsigned short)*cptrconvhelp++ * 3;	/* * 0.59  green */
			grayvalue += (unsigned short)*cptrconvhelp++;	/** 0.11 blue */

			*graydatahelp = (unsigned short)grayvalue;	/* store the grayvalues */
			graydatahelp++;
			if (maxvalue < grayvalue)
			    maxvalue = grayvalue;
			if (minvalue > grayvalue)
			    minvalue = grayvalue;
		    }
		} else {

		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
			grayvalue = (unsigned short)*credptr++ * 2;	/** 0.3 red */
			grayvalue += (unsigned short)*cgreenptr++ * 3;	/* * 0.59 green */
			grayvalue += (unsigned short)*cblueptr++;	/** 0.11 blue */

			*graydatahelp = (unsigned short)grayvalue;	/* store the grayvalues */
			graydatahelp++;
			if (maxvalue < grayvalue)
			    maxvalue = grayvalue;
			if (minvalue > grayvalue)
			    minvalue = grayvalue;
		    }


		}
	    }
	    /********** now put graydata in imagearray **********/
#ifdef DEBUG_IMAGE
	    graydatahelp = graydata;

	    for (ctr = 0; ctr < 250; ctr++) {
		printf(" %d ", *graydatahelp++);
	    }
	    printf("\n");
#endif


	    bytefactor = (((float)(maxvalue - minvalue)) / ((float)(G_ncolors) - (float)graylutoffset));
	    if (bytefactor == 0.0)
		bytefactor = 1.0;

#ifdef DEBUG_IMAGE
	    printf("scalex : %f\n", scalex);
	    printf("scaley : %f\n", scaley);
	    printf("shiftfactor : %d\n", shiftfactor);
	    printf("bytefactor : %f\n", bytefactor);
	    printf("maxvalue : %d\n", maxvalue);
	    printf("minvalue : %d\n", minvalue);
	    printf("8 bit gray data  \n");

#endif


	    yctri = 0.0;
	    ptr = (unsigned short *)graydata;	/* source of convert */


	    ptrconv = (unsigned char *)timages->image;	/* destination of
							 * convert */
	    for (yctr = 0; yctr < image_display_height; yctr++) {
		xctri = 0.0;

		for (xctr = 0; xctr < image_display_width; xctr++) {
		    ptrconvhelp = ptr + (((int)xctri + ((int)yctri) * timage1.Columns));

		    *ptrconv = (unsigned short)graylut[(graylutoffset + ((unsigned short)((float)(*ptrconvhelp - minvalue) /
				    (float)bytefactor)))];
#ifdef DEBUG_IMAGE
		    /*
		     * if ((*ptrconv < graylutoffset) || (*ptrconv >
		     * (G_ncolors))) printf("*ptrconvhelp %d *ptrconv: %d
		     * \n", (int)*ptrconvhelp,(int) *ptrconv); 
		     */
#endif

#ifdef DEBUG_IMAGE

		    /* printf(" %d %d \n",*ptrconvhelp,*ptrconv); */
#endif

		    ptrconv++;

		    xctri += scalex;


		}
		yctri += scaley;
		/*
		 * (printf("yctri : %d %f \n",(int)yctri,yctri); 
		 */
	    }



	    /******/
	    free(graydata);

#ifdef DEBUG_IMAGE

	    printf("\n");
#endif
	} else if (strncmp(timage1.Photometric_Interpretation, "PALETTECOLOR",
          strlen("PALETTECOLOR")) == 0) {


	    /*********************** PALETTECOLOR ***********************/
#ifdef DEBUG_IMAGE
	    printf("PALETTECOLOR PICTURE \n");
#endif

	    if ((graydata = (unsigned short *)calloc((unsigned long)(((unsigned long)timage1.Rows * (unsigned long)timage1.Columns
				+ 2) * sizeof(unsigned short)), sizeof(unsigned char))) == (unsigned short *)NULL)
		malloc_error();


	    if ((graylookup = (unsigned short *)calloc((unsigned long)(((unsigned long)timage1.Red_Palette_Descriptors[0] +
				2) * sizeof(unsigned short)), sizeof(unsigned char))) == (unsigned short *)NULL)
		malloc_error();


	    /*
	     * first i convert the palettes into the gray values. then i
	     * match the pixeldata into the graydatalookuptable witch
	     * correspond with the palettecolor values 
	     */



	    /* scale image, put image in timages, */
	    scalex = (float)timage1.Columns / (float)image_display_width;
	    scaley = (float)timage1.Rows / (float)image_display_height;

	    /* for lookup table */
	    shiftfactor = 0;
	    /*
	     * if stored == 0111 1110,highbit==6,bitsstored == 6,then
	     * shiftfactor =1, data >> 1 !! 
	     */


	    bytefactorl = 0;
	    hibitmask = 1L;	/* for masking the highbit to calc 2
				 * Komplement */

	    for (ctr = 0; ctr < timage1.Red_Palette_Descriptors[2]; ctr++) {
		bytefactorl <<= 1;
		bytefactorl |= 1;
		hibitmask <<= 1;

	    }
	    hibitmask >>= 1;	/* if there are 2 bits stored, then hibit
				 * mask == (long)0000 0010 */
	    bytemask = bytefactorl >> 1;

#ifdef DEBUG_IMAGE
	    printf("bytefactorl : %ld\n", bytefactorl);
#endif

	    /*
	     * first i convert the lookuptables. this data is always type of
	     * short 
	     */
	    if (timage1.Pixel_Representation == 0) {
		ptr = (unsigned short *)timage1.Red_Palette_Data;

		for (ctr = 0; ctr < timage1.Red_Palette_Descriptors[0]; ctr++) {


		    *ptr = (((*ptr)) & bytefactorl);
		    *ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);

		    ptr++;
		}
		ptr = (unsigned short *)timage1.Green_Palette_Data;

		for (ctr = 0; ctr < timage1.Green_Palette_Descriptors[0]; ctr++) {


		    *ptr = (((*ptr)) & bytefactorl);
		    *ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);
		    ptr++;
		}
		ptr = (unsigned short *)timage1.Blue_Palette_Data;

		for (ctr = 0; ctr < timage1.Blue_Palette_Descriptors[0]; ctr++) {


		    *ptr = (((*ptr)) & bytefactorl);
		    *ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);
		    ptr++;
		}

	    } else {
		ptr = (unsigned short *)timage1.Red_Palette_Data;

		for (ctr = 0; ctr < timage1.Red_Palette_Descriptors[0]; ctr++) {


		    *ptr = (((*ptr)) & bytefactorl);
		    *ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
		    ptr++;
		}
		ptr = (unsigned short *)timage1.Green_Palette_Data;

		for (ctr = 0; ctr < timage1.Green_Palette_Descriptors[0]; ctr++) {


		    *ptr = (((*ptr)) & bytefactorl);
		    *ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
		    ptr++;
		}

		ptr = (unsigned short *)timage1.Blue_Palette_Data;

		for (ctr = 0; ctr < timage1.Blue_Palette_Descriptors[0]; ctr++) {


		    *ptr = (((*ptr)) & bytefactorl);
		    *ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
		    ptr++;
		}

	    } /* if timage1.Pixel_Representation == 0 for lookup conversion */

	    /*
	     * now i map the valuerange of the palletcolor to the index of
	     * graydata 
	     */

	    palettemaxvalue = (int)(((float)timage1.Red_Palette_Data[timage1.Red_Palette_Descriptors[0] - 2] * 0.3) + ((float)timage1.Blue_Palette_Data[timage1.Blue_Palette_Descriptors[0] - 2] *
		    0.11) + ((float)timage1.Green_Palette_Data[timage1.Green_Palette_Descriptors[0] - 2] * 0.59));


	    paletteminvalue = (int)(((float)timage1.Red_Palette_Descriptors[1] * 0.3) + ((float)timage1.Green_Palette_Descriptors[1] *
		    0.59) + ((float)timage1.Blue_Palette_Descriptors[1] * 0.11));

	    /* 
	     * palette_to_gray_factor=(float)((float)(timage1.Red_Palette_Descr
	     * iptors[0]-2) -
	     * (float)(timage1.Red_Palette_Descriptors[1]))/((float)((G_ncolor
   s)-graylutoffset)); *//*                                     number of values stored   - first value */


	    palette_to_gray_factor = (float)((float)(palettemaxvalue) - (float)(paletteminvalue)) / ((float)((G_ncolors) - graylutoffset));
	    for (ctr = 0; ctr < (timage1.Red_Palette_Descriptors[0]); ctr++) {

		graylookup[ctr] = graylutoffset + (unsigned short)((float)timage1.Red_Palette_Data[ctr] * 0.3 + (float)timage1.Green_Palette_Data[ctr] *
		    0.59 + (float)timage1.Blue_Palette_Data[ctr] * 0.11 + 0.5);

#ifdef DEBUG_IMAGE
		printf("g %d ", graylookup[ctr]);
#endif

	    }


#ifdef DEBUG_IMAGE
	    printf("\n palettemaxvalue : %d\n", palettemaxvalue);
	    printf("paletteminvalue : %d\n", paletteminvalue);
	    printf("palette_to_gray_factor : %f\n", palette_to_gray_factor);
#endif

	    /* values for pixeldata */
	    shiftfactor = (timage1.High_Bit + 1) - timage1.Bits_Stored;
	    /*
	     * if stored == 0111 1110,highbit==6,bitsstored == 6,then
	     * shiftfactor =1, data >> 1 !! 
	     */


	    bytefactorl = 0;
	    hibitmask = 1L;	/* for masking the highbit to calc 2
				 * Komplement */

	    for (ctr = 0; ctr < timage1.Bits_Stored; ctr++) {
		bytefactorl <<= 1;
		bytefactorl |= 1;
		hibitmask <<= 1;

	    }
	    hibitmask >>= 1;	/* if there are 2 bits stored, then hibit
				 * mask == (long)0000 0010 */
	    bytemask = bytefactorl >> 1;

#ifdef DEBUG_IMAGE
	    printf("bytefactorl : %ld\n", bytefactorl);
#endif

	    if (timage1.Bits_Stored > 8) {
		ptr = (unsigned short *)timage1.Pixel_Data;

		maxvalue = 0;
		minvalue = 0xFFFF;

		if (timage1.Pixel_Representation == 0) {
#ifdef DEBUG_IMAGE
		    printf("Unsigned value \n");
		    printf("signed value mask 1 bytemask,%lx hibitmask %lx  bytefactorl %lx\n", bytemask, hibitmask,
			bytefactorl);

#endif


		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {


			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & 0x8000) == 0x8000) ? (*ptr - 0x10000) : (*ptr);
			ptr++;
		    }


		} else {

		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {

			*ptr = (((*ptr) >> shiftfactor) & bytefactorl);
			*ptr = ((*ptr & hibitmask) == 0L) ? ((*ptr & bytemask) + hibitmask) : (*ptr & bytemask);
			ptr++;
		    }



		}

		/* now convert the palletecolor values in grayvalues */

		ptr = (unsigned short *)timage1.Pixel_Data;
		ptrconvhelp = ptr;

		graydatahelp = graydata;	/* destination of rgb to gray
						 * converted array */

		maxvalue = 0;
		minvalue = 0xFFFF;

		for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {


		    /*
		     * grayvalue=(unsigned short)
		     * ((((float)*ptrconvhelp++)-((float)(timage1.Red_Palette_
		     *
		     * Descriptors[1])))/palette_to_gray_factor)+graylutoffset
		     * ; 
		     */
		    grayvalue = (unsigned short)(((float)graylookup[*ptrconvhelp++]) / palette_to_gray_factor) + graylutoffset;
		    grayvalue = MIN(MAX(grayvalue, graylutoffset), (G_ncolors));
		    *graydatahelp = grayvalue;	/* store the grayvalues and
						 * cut range */
		    graydatahelp++;
		    if (maxvalue < grayvalue)
			maxvalue = grayvalue;
		    if (minvalue > grayvalue)
			minvalue = grayvalue;

		}


	    }
	    /* 16 bit */
	    else {
		/* *******************its a 8 bit picture */


		cptr = (unsigned char *)timage1.Pixel_Data;

		if (timage1.Pixel_Representation == 0) {
#ifdef DEBUG_IMAGE
		    printf("Unsigned value timage1.Rows * timage1.Columns : %d \n", timage1.Rows * timage1.Columns);
#endif
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
#ifdef DEBUG_IMAGE

			/*
			 * if((ctr < 250) || (*cptr > 50))printf(" %d
			 * ",*cptr); 
			 */
#endif

			*cptr = ((((*cptr)) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & 0x80) == 0x80) ? (*cptr - 0x100) : (*cptr);

			cptr++;
		    }

		} else {	/* 2 komplement */
		    for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {

			*cptr = (((*cptr) >> shiftfactor) & (unsigned char)bytefactorl);
			*cptr = ((*cptr & hibitmask) == 0L) ? ((*cptr & bytemask) + hibitmask) : (*cptr &
			    bytemask);
			cptr++;
		    }
		}





		/* now convert the palletecolor values in grayvalues */

		cptr = (unsigned char *)timage1.Pixel_Data;
		cptrconvhelp = cptr;

		graydatahelp = graydata;	/* destination of rgb to gray
						 * converted array */

		maxvalue = 0;
		minvalue = 0xFF;

		for (ctr = 0; ctr < (timage1.Rows * timage1.Columns); ctr++) {
		    /*
		     * grayvalue=(unsigned short)
		     * ((((float)*cptrconvhelp++)-((float)(timage1.Red_Palette
		     *
		     * _Descriptors[1])))/palette_to_gray_factor)+graylutoffse
		     * t; 
		     */
		    grayvalue = (unsigned short)(((float)graylookup[*cptrconvhelp++]) / palette_to_gray_factor) +
			graylutoffset;

#ifdef DEBUG_IMAGE

		    if (ctr < 25)
			printf(" %d %d ", grayvalue, *cptrconvhelp);
#endif

		    grayvalue = MIN(MAX(grayvalue, graylutoffset), (G_ncolors));
		    *graydatahelp = grayvalue;	/* store the grayvalues and
						 * cut range */
		    graydatahelp++;
		    if (maxvalue < grayvalue)
			maxvalue = grayvalue;
		    if (minvalue > grayvalue)
			minvalue = grayvalue;

		}

#ifdef DEBUG_IMAGE
		graydatahelp = graydata;

		for (ctr = 0; ctr < 25; ctr++) {
		    printf(" %d ", *graydatahelp++);
		}
		printf("\n");
#endif



	    } /* else its 8  bit data */


	    bytefactor = (((float)(maxvalue - minvalue)) / ((float)(G_ncolors) - (float)graylutoffset));
	    if (bytefactor == 0.0)
		bytefactor = 1.0;


#ifdef DEBUG_IMAGE
	    printf("scalex : %f\n", scalex);
	    printf("scaley : %f\n", scaley);
	    printf("shiftfactor : %d\n", shiftfactor);
	    printf("bytefactor : %f\n", bytefactor);
	    printf("maxvalue : %d\n", maxvalue);
	    printf("minvalue : %d\n", minvalue);
	    printf("8 bit gray data  \n");

#endif

	    yctri = 0.0;
	    ptr = (unsigned short *)graydata;	/* source of convert */


	    ptrconv = (unsigned char *)timages->image;	/* destination of
							 * convert */
	    for (yctr = 0; yctr < image_display_height; yctr++) {
		xctri = 0.0;

		for (xctr = 0; xctr < image_display_width; xctr++) {
		    ptrconvhelp = ptr + (((int)xctri + ((int)yctri) * timage1.Columns));

		    *ptrconv = (unsigned short)graylut[(graylutoffset + ((unsigned short)((float)(*ptrconvhelp - minvalue) /
				    (float)bytefactor)))];



		    ptrconv++;

		    xctri += scalex;


		}
		yctri += scaley;
		/*
		 * (printf("yctri : %d %f \n",(int)yctri,yctri); 
		 */
	    }

	    free(graydata);

	}
	/* if PALETTECOLOR */
	else {			/********* not implemented ******/

	    /* i dont no what to do */
#ifdef DEBUG_IMAGE
	    printf("DON`T Know Image Format : %s \n", timage1.Photometric_Interpretation);
#endif

	    ptrconv = (unsigned char *)timages->image;	/* destination of
							 * convert */
	    for (yctr = 0; yctr < image_display_height; yctr++) {

		for (xctr = 0; xctr < image_display_width; xctr++) {

		    *ptrconv = (unsigned short)graylutoffset;
		    ptrconv++;

		}
	    }

	} /* end if chains */

    } /* end if(MULTI); */
    if ((MULTI) && (CalcStatus == ICALC))
	return;			/* rest will be done by father process */

    strcpy(timages->Patname, timage1.Patname);
    strcpy(timages->SOP, timage1.SOP);
    strcpy(timages->Study_ID, timage1.Study_ID);
    strcpy(timages->Modality, timage1.Modality);

    timages->AssociationLevel = element.Q_DSU_AssociationLevel;
    timages->AssociationID = element.Q_AssociationID;
    timages->xi = timage1.xi;
    timages->window = timage1.window;

    if (element.Q_DSU_Direction == Incoming) {
	sprintf(tempstring, "%s%s%s\n", element.Q_peerAE, RIGHTDIRECTION, element.Q_myAE);
    } else {
	sprintf(tempstring, "%s%s%s\n", element.Q_peerAE, LEFTDIRECTION, element.Q_myAE);
    }
    if (element.Q_DSU_AssociationLevel == NoAssociation)
	sprintf(tempstring, "%s", element.Q_peer);

    strcpy(timages->AE_Titles, tempstring);
    /*
     * printf("in convert  %s \n",tempstring); 
     */
    return;
}

/*******************************************************************/
void
DestroyImage(Images_M * images)
{
#ifdef DEBUG_IMAGE
    printf("DESTROY x %d y %d \n", images->window_x, images->window_y);
#endif

    XClearArea(G_display, G_big_window, images->window_x, images->window_y, IMAGESWIDTH, IMAGESHEIGHT, False);

    if (images->active == IMAGEACTIVE) {
	XUnmapWindow(G_display, images->window);
	XDestroyWindow(G_display, images->window);
	images->xi->data = NULL;/* imagememory not to give free ! */
	XDestroyImage(images->xi);
    }
    XFlush(G_display);

    return;
}

/*******************************************************************/
void CTNDISP_CreateShow()
{
    /*
     * here I`ll do the initializing stuff, that will setup the transfer and
     * image stuctures 
     */

    unsigned long valuemask;

    XSetWindowAttributes
	attributes;

    char temp[MAX_STRING_LENGTH];
    int w;
    int counter, ycounter, scounter;

    int width, height, widthtmp, heighttmp;
    unsigned char *ptr=NULL;


    height = G_display_height;
    width = G_display_width;

    /* subtract borders */

    width -= (IMAGE_X_OFFSET * 2 + IMAGE_SHOW_SPACE * 2 + 4);	/* 4 for nicer reasons */
    if ((G_display_height > (DisplayHeight(G_display, G_screen) - 30))) {
	height -= (IMAGE_Y_OFFSET + 30);	/* boarder offset if window
						 * is maximum */
    } else {
	height -= IMAGE_Y_OFFSET;
    }


    widthtmp = (int)(((float)width /*-(IMAGE_SHOW_SPACE * 2)*/ ) / 20.0 + 0.5);	/* calc weidth of areas */
    heighttmp = (int)((float)height / (float)MAXWINDOWS);	/* calc y areas */

    /* calc the size of the show windows */
    show_width = 6 * widthtmp;	/* see below */
    show_height = heighttmp - SHOW_SPACE;
    text_height = (int)((float)show_height / 3.0 + 0.5);
    image_text_height = (int)((float)(image_text_offset - 2) / 3.0 + 0.5);
    LINE_THICKNESS = (int)((float)show_height / 3.0 - 2.0 + 0.5);
    /* if the display is very small */
    if ((text_height < 8) && (show_height >= 16)) {
	text_height = (int)((float)(show_height - 2) / 2.0);
	LINE_THICKNESS = 2;
    }
    /* calc the percentage line sizes */
    if (LINE_THICKNESS < 1)
	LINE_THICKNESS = 1;
    LINE_OFFSET = (int)((float)show_height - (float)LINE_THICKNESS / 2.0);
    if (LINE_OFFSET > (show_height - 1))
	LINE_OFFSET = show_height - 1;
    XSetLineAttributes(G_display, G_gcgreen, LINE_THICKNESS, LineSolid, CapRound, JoinMiter);
#ifdef DEBUG_IMAGE
    printf("text_height %d show_height %d image_text_height %d image_text_offset %d \n", text_height, show_height, image_text_height, image_text_offset);
#endif
    IMAGESWIDTH = 7 * widthtmp;
    IMAGESHEIGHT = IMAGESWIDTH + image_text_offset	/* 7*heighttmp-image_text
	    _offset */ ;
    if ((IMAGESHEIGHT * (IMAGES / 2)) > (heighttmp * MAXWINDOWS))
	IMAGESHEIGHT = ((heighttmp * MAXWINDOWS) - IMAGE_SHOW_SPACE) / (IMAGES / 2)	/* +(image_text_offset*(I
	        MAGES/2)) */ ;
    if ((IMAGESHEIGHT - image_text_offset) > IMAGESWIDTH) {
	IMAGESHEIGHT = IMAGESWIDTH + image_text_offset;
    } else {
	IMAGESWIDTH = IMAGESHEIGHT - image_text_offset;
    }

    show_width = width - IMAGESWIDTH * 2;

    /* show_width=6*heighttmp; */
#ifdef DEBUG_IMAGE
    printf("iw %d ih %d  ,space %d \n", IMAGESWIDTH, IMAGESHEIGHT, IMAGESWIDTH * IMAGESHEIGHT);
#endif

    SHOW_X_OFFSET = IMAGE_X_OFFSET + IMAGE_SHOW_SPACE + IMAGESWIDTH;
    /* left up */
    IMAGEWINDOW1X = IMAGE_X_OFFSET;
    IMAGEWINDOW1Y = IMAGE_Y_OFFSET;

    /* right up ,  Y is calculatet to be in line with node windows */
    IMAGEWINDOW2X = IMAGE_X_OFFSET + IMAGESWIDTH + 2 * IMAGE_SHOW_SPACE + show_width;
    IMAGEWINDOW2Y = IMAGE_Y_OFFSET;
    /* left down */
    IMAGEWINDOW3X = IMAGE_X_OFFSET;
    IMAGEWINDOW3Y = SHOW_Y_OFFSET + (MAXWINDOWS * (show_height + SHOW_SPACE)) - IMAGESHEIGHT - SHOW_SPACE;
    /* right down */
    IMAGEWINDOW4X = IMAGE_X_OFFSET + IMAGESWIDTH + 2 * IMAGE_SHOW_SPACE + show_width;
    IMAGEWINDOW4Y = SHOW_Y_OFFSET + (MAXWINDOWS * (show_height + SHOW_SPACE)) - IMAGESHEIGHT - SHOW_SPACE;
#ifdef DEBUG_IMAGE
    printf("%d %d %d %d \n", IMAGEWINDOW1X, IMAGEWINDOW1Y, IMAGEWINDOW4X, IMAGEWINDOW4Y);
#endif
    IMAGESHEIGHT -= image_text_offset;

    for (counter = 0; counter < MAXWINDOWS; counter++) {
	show_list[counter].element.Q_AssociationID = FREE;

	for (ycounter = 0; ycounter < 20; ycounter++) {
	    strcpy(show_list[counter].show_strings[ycounter], "\0");
	}
    }



    scounter = 0;
    for (ycounter = 0; ycounter < MAXWINDOWS; ycounter++) {
	valuemask = CWBackPixel;
	attributes.background_pixel = graylut[G_ncolors - 1];


	show_list[scounter++].show_window = XCreateWindow(G_display,
	    G_big_window,
	    SHOW_X_OFFSET + (SHOW_SPACE),
	    SHOW_Y_OFFSET + (SHOW_SPACE + show_height) * ycounter,
	    show_width, show_height,
	    0,
	    CopyFromParent,
	    InputOutput,
	    CopyFromParent,
	    valuemask,
	    &attributes);


    }



    for (counter = 0; counter < MAXWINDOWS; counter++) {

	XMapWindow(G_display, show_list[counter].show_window);
    }




    /*
     * And...display the nodename and labels while we are here... 
     */
    strcpy(temp, "Node: ");
    strcat(temp, G_nodename);
    w = XTextWidth(fontLG, temp, strlen(temp));
    XDrawString(G_display, G_big_window, G_gclargeT,
	(G_display_width / 2) - (w / 2), NODE_Y_OFFSET,
	temp, strlen(temp));
    /* 
     * sprintf( temp, "Jens.Burkert@arbi.informatik.uni-oldenburg.de"); w =
     * XTextWidth( fontSM, temp, strlen(temp) ); XDrawString (G_display,
     * show_list[0].show_window, G_gcsmallT, 0, 16, temp, strlen (temp));	 */

    XFlush(G_display);

    /* init the image struct parameters */

    for (counter = 0; counter < IMAGES; counter++) {
	strcpy(images[counter].Patname, "");
	strcpy(images[counter].AE_Titles, "");
	strcpy(images[counter].Study_ID, "");
	strcpy(images[counter].Modality, "");
	strcpy(images[counter].SOP, "");
	images[counter].active = IMAGEINACTIVE;
	images[counter].AssociationLevel = 0;
	images[counter].AssociationID = 0;
	/* give the images some space */
	if ((ptr = (unsigned char *)calloc((unsigned)(IMAGESWIDTH * IMAGESHEIGHT + 2), sizeof(char))) ==
	    (unsigned char *)NULL)
	    malloc_error();
	images[counter].image = ptr;
#ifdef DEBUG_IMAGE
	printf("create images[counter].image : %ld\n", images[counter].image);
#endif

    }
#ifdef DEBUG_IMAGE
    printf("iw %d ih %d  ,space %d \n", IMAGESWIDTH, IMAGESHEIGHT, IMAGESWIDTH * IMAGESHEIGHT);
#endif
    *ptr = 0;
    if (IMAGES > 0)
	images[0].window_x = IMAGEWINDOW1X;
    if (IMAGES > 0)
	images[0].window_y = IMAGEWINDOW1Y;

    if (IMAGES > 1)
	images[1].window_x = IMAGEWINDOW2X;
    if (IMAGES > 1)
	images[1].window_y = IMAGEWINDOW2Y;

    if (IMAGES > 2)
	images[2].window_x = IMAGEWINDOW3X;
    if (IMAGES > 2)
	images[2].window_y = IMAGEWINDOW3Y;

    if (IMAGES > 3)
	images[3].window_x = IMAGEWINDOW4X;
    if (IMAGES > 3)
	images[3].window_y = IMAGEWINDOW4Y;

    if (MULTI) {
	ishmID = shmget(ishmkey, IMAGESWIDTH * IMAGESHEIGHT + 2, IPC_CREAT | 0777);
	if (ishmID == -1) {
	    shared_error();
	    exit(-1);
	}
	/* get shared segment addres in ishmaddr */
	ishmaddr = (unsigned char *)NULL;
	if ((ishmaddr = (unsigned char *)shmat(ishmID, (char*)ishmaddr, SHM_RND)) == (unsigned char *)-1) {
	    shmctl(ishmID, IPC_RMID, shm_buf);

	    shared_error();
	    exit(-1);
	}
    }
    return;
}

/*******************************************************************/
void CTNDISP_RefreshStatus()
{
    char
     temp[MAX_STRING_LENGTH];
    int
     w, counter, ycounter;
    unsigned long charheighta, charheightd, charheight;

    /*
     * Update the node name.... 
     */
    strcpy(temp, "Node: ");
    strcat(temp, G_nodename);
    w = XTextWidth(fontLG, temp, strlen(temp));
    XDrawString(G_display, G_big_window, G_gclargeT,
	(G_display_width / 2) - (w / 2), NODE_Y_OFFSET,
	temp, strlen(temp));

    XGetFontProperty(fontSM, XA_STRIKEOUT_ASCENT, &charheighta);
    XGetFontProperty(fontSM, XA_STRIKEOUT_DESCENT, &charheightd);
    charheight = charheighta + charheightd;
    /* printf("CHARHEIGHT %ld %ld %ld\n",charheight,charheighta,charheightd); */
    for (counter = 0; counter < MAXWINDOWS; counter++) {
	XClearArea(G_display, show_list[counter].show_window, 0, 0, 0, 0, False);

	for (ycounter = 0; ycounter < 2; ycounter++) {
	    strcpy(temp, show_list[counter].show_strings[ycounter]);

	    w = XTextWidth(fontSM, temp, strlen(temp));
	    XDrawString(G_display, show_list[counter].show_window, G_gcsmallT,
		00,
		(ycounter + 1) * text_height,	/* +1 to set baseline */
		temp, strlen(temp));
	}
	if (show_list[counter].element.Q_DSU_OperationState == RunningOperation) {
	    if (show_list[counter].element.Q_DSU_Percent > 100) {
		show_list[counter].element.Q_DSU_Percent = 100;
	    }
	    if (show_list[counter].element.Q_DSU_Percent < 0) {
		show_list[counter].element.Q_DSU_Percent = 00;
	    }
	    if (show_list[counter].element.Q_DSU_Percent > 0) {

		if (show_list[counter].element.Q_DSU_Direction == Incoming) {
		    XDrawLine(G_display, show_list[counter].show_window, G_gcgreen, 0, LINE_OFFSET, (int)(show_width
			    * show_list[counter].element.Q_DSU_Percent) / 100, LINE_OFFSET);
		} else {
		    XDrawLine(G_display, show_list[counter].show_window, G_gcgreen, show_width - (int)(show_width
			    * show_list[counter].element.Q_DSU_Percent) / 100, LINE_OFFSET, show_width, LINE_OFFSET);

		}
	    }
	}
    }
    XFlush(G_display);
    return;
}
/*******************************************************************/
void
CTNDISP_RefreshWindowArea(int windowarea)
{
    char
     temp[MAX_STRING_LENGTH];
    int
     w, ycounter;
    unsigned long charheighta, charheightd, charheight;

    /*
     * Update the node name.... 
     */
    /*
     * strcpy(temp, "Node: "); strcat(temp, G_nodename); w =
     * XTextWidth(fontLG, temp, strlen(temp)); 
     *
     * XDrawString(G_display, G_big_window, G_gclargeT, (G_display_width / 2) -
     * (w / 2), NODE_Y_OFFSET, temp, strlen(temp)); 
     */
    if ((windowarea < 0) || (windowarea >= MAXWINDOWS)) {
	return;
    }
    XGetFontProperty(fontSM, XA_STRIKEOUT_ASCENT, &charheighta);
    XGetFontProperty(fontSM, XA_STRIKEOUT_DESCENT, &charheightd);
    charheight = charheighta + charheightd;
    /*
     * printf("CHARHEIGHT %ld %ld %ld\n",charheight,charheighta,charheightd); 
     */
/*
	XClearArea(G_display, show_list[windowarea].show_window, 0, 0, 0, 0, False);
*/
    if ((show_list[windowarea].element.Q_DSU_OperationState == StopOperation) || (show_list[windowarea].element.Q_DSU_OperationState == StartOperation)) {
	XClearArea(G_display, show_list[windowarea].show_window, 0, 0, 0, 0, False);
	/* only refres when stop or start */
	for (ycounter = 0; ycounter < 2; ycounter++) {
	    strcpy(temp, show_list[windowarea].show_strings[ycounter]);

	    w = XTextWidth(fontSM, temp, strlen(temp));
	    XDrawString(G_display, show_list[windowarea].show_window, G_gcsmallT,
		00,
		(ycounter + 1) * text_height,	/* +1 to set baseline */
		temp, strlen(temp));
	}
    }
    if (show_list[windowarea].element.Q_DSU_OperationState == RunningOperation) {
	if (show_list[windowarea].element.Q_DSU_Percent > 100) {
	    show_list[windowarea].element.Q_DSU_Percent = 100;
	}
	if (show_list[windowarea].element.Q_DSU_Percent < 0) {
	    show_list[windowarea].element.Q_DSU_Percent = 00;
	}
	if (show_list[windowarea].element.Q_DSU_Percent > 0) {

	    if (show_list[windowarea].element.Q_DSU_Direction == Incoming) {
		XDrawLine(G_display, show_list[windowarea].show_window, G_gcgreen, 0, LINE_OFFSET, (int)(show_width
			* show_list[windowarea].element.Q_DSU_Percent) / 100, LINE_OFFSET);
	    } else {
		XDrawLine(G_display, show_list[windowarea].show_window, G_gcgreen, show_width - (int)(show_width
			* show_list[windowarea].element.Q_DSU_Percent) / 100, LINE_OFFSET, show_width, LINE_OFFSET);

	    }
	}
    }
    XFlush(G_display);
    return;
}



/*
 * Remember this for future reference
 *
	char    *fontname1 = 
			"-adobe-helvetica-medium-r-normal--14-*";
	char	*fontname2 = 
		"-adobe-helvetica-bold-r-normal-*-24-*-*-*-*-*-*-*";
*/

#ifdef ALTERNATE_FONTS

#define FONT_SM_A	"-adobe-helvetica-*-r-normal--14-*"
#define FONT_LG_A	"-adobe-*-*-r-normal--24-*"

#define FONT_SM_B	"-*-*-*-*-*--10-*"
#define FONT_LG_B	"-*-*-*-*-*--16-*"

#else

#define FONT_SM_A 	"-adobe-helvetica-medium-r-normal--14-*"
#define FONT_SM_AB 	"-adobe-helvetica-bold-r-normal--14-*"
#define FONT_LG_A	"-adobe-helvetica-bold-r-normal--24-*"

#define FONT_SM_B 	"-adobe-helvetica-medium-r-normal--10-*"
#define FONT_LG_B	"-adobe-helvetica-medium-r-normal--18-*"

#define FONT_SM_C 	"-adobe-helvetica-medium-r-normal--12-*"
#define FONT_LG_C	"-adobe-helvetica-medium-r-normal--18-*"

#endif

/*------------------------ shared_error --------------*/
void 
shared_error()
{
    fprintf(stderr, "Not enough Memory for Shared memory! Fatal Error !\n");
    fprintf(stderr, "Try running ctndisp with -noMulti !\n");
    fprintf(stderr, "Please check with ipcs/ipcrm if there is shared memory left.\n");
    exit(-1);
}


/*******************************************************************/
int CTNDISP_CreateGC()
{
    unsigned long valuemask;
    XColor color;
    XGCValues values;
    char tc_small[MAX_STRING_LENGTH], tc_large[MAX_STRING_LENGTH], tc_smallbold[MAX_STRING_LENGTH];
    Font fontsmall, fontlarge, fontsmallbold;

    if ((G_display_height <= (DisplayHeight(G_display, G_screen) / 2)) && (G_display_width <= DisplayWidth(G_display, G_screen) / 2)) {
	strcpy(tc_small, FONT_SM_B);
	strcpy(tc_large, FONT_LG_B);
	strcpy(tc_smallbold, FONT_SM_B);
	image_text_offset = 8 * 3 + 6;
    } else {
	strcpy(tc_small, FONT_SM_A);	/* oiginal SM_A and LG_B */
	strcpy(tc_smallbold, FONT_SM_AB);
	strcpy(tc_large, FONT_LG_C);
	image_text_offset = 13 * 3 + 3;

    }
    if ((fontsmall = XLoadFont(G_display, tc_small)) == 0) {
	fprintf(stderr, "CTNDISP_CreateGC: Server does not support small font.\n");
	return 0;
    }
    if ((fontsmallbold = XLoadFont(G_display, tc_smallbold)) == 0) {
	fontsmallbold = fontsmall;
    }
    if ((fontlarge = XLoadFont(G_display, tc_large)) == 0) {
	fprintf(stderr, "CTNDISP_CreateGC: Server does not support large font.\n");
	return 0;
    }
    fontSM = XQueryFont(G_display, fontsmall);
    fontSMbold = XQueryFont(G_display, fontsmallbold);
    fontLG = XQueryFont(G_display, fontlarge);

    valuemask = GCForeground | GCBackground | GCFont | GCFillStyle;
    values.font = fontlarge;

    XParseColor(G_display, G_cmap, "white", &color);
    XAllocColor(G_display, G_cmap, &color);
    values.background = color.pixel;

    XParseColor(G_display, G_cmap, "black", &color);
    XAllocColor(G_display, G_cmap, &color);
    values.foreground = color.pixel;

    values.fill_style = FillSolid;

    if ((G_gclargeT = XCreateGC(G_display, RootWindow(G_display, G_screen),
		valuemask, &values)) == NULL) {
	fprintf(stderr, "CTNDISP_CreateGC: Cannot create Large Text Graphics Context\n");
	return 0;
    }
    values.font = fontsmall;


    if ((G_gcsmallT = XCreateGC(G_display, RootWindow(G_display, G_screen),
		valuemask, &values)) == NULL) {
	fprintf(stderr, "CTNDISP_CreateGC: Cannot create Small Text Graphics Context\n");
	return 0;
    }
    values.font = fontsmallbold;


    if ((G_gcsmallTbold = XCreateGC(G_display, RootWindow(G_display, G_screen),
		valuemask, &values)) == NULL) {
	fprintf(stderr, "CTNDISP_CreateGC: Cannot create Small Text Graphics Context\n");
	return 0;
    }
    /*
     * Create the colors we need... 
     */
    values.font = fontlarge;
    values.background = values.foreground;
    XParseColor(G_display, G_cmap, "black", &color);
    XAllocColor(G_display, G_cmap, &color);
    values.foreground = color.pixel;
    if ((G_gcblack = XCreateGC(G_display, RootWindow(G_display, G_screen),
		valuemask, &values)) == NULL) {
	fprintf(stderr, "CTNDISP_CreateGC: Cannot create Black Graphics Context\n");
	return 0;
    }
    valuemask = GCForeground | GCBackground | GCFont | GCFillStyle;
    values.font = fontlarge;

    XParseColor(G_display, G_cmap, "red", &color);
    XAllocColor(G_display, G_cmap, &color);
    values.foreground = color.pixel;

    XParseColor(G_display, G_cmap, "black", &color);
    XAllocColor(G_display, G_cmap, &color);
    values.background = color.pixel;

    values.fill_style = FillSolid;


    if ((G_gcred = XCreateGC(G_display, RootWindow(G_display, G_screen),
		valuemask, &values)) == NULL) {
	fprintf(stderr, "CTNDISP_CreateGC: Cannot create Red Graphics Context\n");
	return 0;
    }
    XParseColor(G_display, G_cmap, "green", &color);
    XAllocColor(G_display, G_cmap, &color);
    values.foreground = color.pixel;

    if ((G_gcgreen = XCreateGC(G_display, RootWindow(G_display, G_screen),
		valuemask, &values)) == NULL) {
	fprintf(stderr, "CTNDISP_CreateGC: Cannot create Green Graphics Context\n");
	return 0;
    }
    return 1; /* success */
}


#define PEEK(array, row, col, ncol) ((array)+(((row)*(ncol))+(col)))

/*****************/
void 
Set_Images_and_Show(int i, int s)
{				/* is callad from main switch in ctndisp */
    IMAGES = i;
    MAXWINDOWS = s;
    return;

}


/*****************/
 
typedef struct _QElementQueue {
    QUEUE_Element qelem;
    struct _QElementQueue *next;
} QElementQueue;

static QElementQueue *imageOpQueue = NULL;
static QElementQueue *imageOpQueueTail = NULL;

void
ImageQueue(QUEUE_Element * tempelement, int command)
{
    /*
     * this will store and give back uncalculated image display commands if
     * MULTI is TRUE 
     */
    QElementQueue *tmpEntry = NULL;

    switch (command) {
    case GET:
	if (imageOpQueue == NULL) {
	    tempelement->Q_DSU_Operation = -1;
	    fprintf(stderr, "Tried to get a Element in ImageQueue when Queue is empty!\n");
	} else {
	    /* remove from queue (head of list) */
	    *tempelement = imageOpQueue->qelem;
	    tmpEntry = imageOpQueue;
	    imageOpQueue = imageOpQueue->next;
	    free(tmpEntry);
	    if (imageOpQueue == NULL) {
		imageOpQueueTail = NULL;
	    }
	}
	break;
    case PUT:
	tmpEntry = (QElementQueue *) malloc(sizeof(*tmpEntry));
	if (tmpEntry == NULL) {
	    fprintf(stderr, "ctndisp: ImageQueue: malloc failure\n");
	} else {
	    /* add to queue (tail of list) */
	    tmpEntry->qelem = *tempelement;
	    tmpEntry->next = NULL;
	    if (imageOpQueueTail == NULL) {
		imageOpQueue = imageOpQueueTail = tmpEntry;
	    } else {
		imageOpQueueTail->next = tmpEntry;
		imageOpQueueTail = tmpEntry;
	    }
	}
	break;
    default:
	fprintf(stderr, "ctndisp: ImageQueue: unknown command: %d\n",
	    command);
	break;
    }

    if (imageOpQueue != NULL) {
	ImageQueueStatus = NOTEMPTY;
    } else {
	ImageQueueStatus = EMPTY;
    }

}

#endif /* HAVE_SHMxxx */
#endif /* X_DISPLAY_MISSING */
