/*
          Copyright (C) 1993, RSNA and Washington University

          The software and supporting documentation for the Radiological
          Society of North America (RSNA) 1993 Digital Imaging and
          Communications in Medicine (DICOM) Demonstration were developed
          at the
                  Electronic Radiology Laboratory
                  Mallinckrodt Institute of Radiology
                  Washington University School of Medicine
                  510 S. Kingshighway Blvd.
                  St. Louis, MO 63110
          as part of the 1993 DICOM Central Test Node project for, and
          under contract with, the Radiological Society of North America.

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER RSNA NOR
          WASHINGTON UNIVERSITY MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          jointly owned by RSNA and Washington University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include the three paragraphs of
          the copyright notice.
*/

/*
 * snq.h
 *
 * snq.h,v
 * Revision 1.2  1996/04/18 15:29:49  meichel
 * Version for CAR'96
 *
 * Revision 1.1  1996/03/28 17:08:01  meichel
 * first commit with new file structure
 *
 * Revision 2.1  1996/03/25 16:50:54  meichel
 * corrected spelling of two functions
 *
 * Revision 2.0  1994/11/17 09:45:41  meichel
 * Version 2.0 for RSNA 1994 demonstration.
 * New features: Metaheader support, new SOP classes, lots of bugs fixed.
 *
 * Revision 1.4  1994/03/17  09:28:39  hewett
 * Updated copyright notice.
 *
 * Revision 1.3  1994/03/16  15:58:45  hewett
 * Added parameter definitions to function prototypes.
 *
 * Revision 1.2  1993/10/14  09:39:45  hewett
 * Updated original MIR header for 1993
 *
 * Revision 1.1  1993/08/31  18:41:01  jens
 * Initial Revision. SNQ Module header.
 *
 * Revision 1.1.1.1  1993/06/04  10:12:45  dicom3
 * Original MIR Release
 *
 * Revision 1.2  1992/08/03  18:15:20  smm
 * Add snq.h,v
 * Add Revision 1.2  1996/04/18 15:29:49  meichel
 * Add Version for CAR'96
 * Add
 * Add Revision 1.1  1996/03/28 17:08:01  meichel
 * Add first commit with new file structure
 * Add
 * Add Revision 2.1  1996/03/25 16:50:54  meichel
 * Add corrected spelling of two functions
 * Add
 * Add Revision 2.0  1994/11/17 09:45:41  meichel
 * Add Version 2.0 for RSNA 1994 demonstration.
 * Add New features: Metaheader support, new SOP classes, lots of bugs fixed.
 * Add
 * Revision 1.4  1994/03/17  09:28:39  hewett
 * Updated copyright notice.
 *
 * Revision 1.3  1994/03/16  15:58:45  hewett
 * Added parameter definitions to function prototypes.
 *
 * Revision 1.2  1993/10/14  09:39:45  hewett
 * Updated original MIR header for 1993
 *
 * Revision 1.1  1993/08/31  18:41:01  jens
 * Initial Revision. SNQ Module header.
 *
 * Revision 1.1.1.1  1993/06/04  10:12:45  dicom3
 * Original MIR Release
 *
 * snq.h,v 1.2 1996/04/18 15:29:49 meichel Exp
 */

#ifndef _SNQ_IS_IN
#define _SNQ_IS_IN

#include "osconfig.h"

/* DEFINITIONS OF CONSTANTS */
#define		MAX_STRING_LENGTH	256 /* same as DSU_MAXSTRINGLEN*/
#define		QUEUE_DIRECTORY		"QUEUE_DIRECTORY"
#define		QUEUE_FILESUFFIX	"snq.dat"
#define		MAX_NAMED_QUEUES	100

/*
 * Typedef the queue structs
 */ 
typedef	struct _QUEUE_Element{
	int     Q_DSU_UserType;
	int     Q_DSU_AssociationType;
	int     Q_DSU_AssociationLevel;
	int     Q_DSU_Operation;
	int     Q_DSU_OperationState;
	int     Q_DSU_Percent;
	int     Q_DSU_Direction;
	char 	Q_active;
   	char 	Q_peer[MAX_STRING_LENGTH];
	char 	Q_peerAE[MAX_STRING_LENGTH];
	char 	Q_myAE[MAX_STRING_LENGTH];
        char    Q_ImageName[MAX_STRING_LENGTH];  /*image name of aktion */
        int     Q_QueueID;    /* the number of the Queue */ 
        int     Q_AssociationID; /* Number of the process for Association ident.*/
}QUEUE_Element;

/*
 * Define the pointers to the queue
 */
typedef struct _QUEUE_Pointers{
	int	SNQ_head,
		SNQ_tail,
		SNQ_size; 
}QUEUE_Pointers;


/*
 * Error codes for functions. 
 * We don't use the condition library and FORM_COND any more.
 */

typedef enum
{
  SNQ_NORMAL = 0, 
  SNQ_QUEUEFULL = 1,
  SNQ_QUEUEEMPTY = 2,
  SNQ_SHAREDMEMORYFAIL = 3,
  SNQ_SEMAPHOREFAIL = 4,
  SNQ_FILEACCESSFAIL = 5,
  SNQ_NOMEMORY = 6,
  SNQ_UNIMPLEMENTED = 7
} SNQ_code;

/*
 * Function Prototypes
 */

SNQ_code SNQ_InitQueue (int qid, int size);
SNQ_code SNQ_KillQueue (int qid);
SNQ_code SNQ_Enqueue (QUEUE_Element *element);
SNQ_code SNQ_Dequeue (QUEUE_Element *element);
SNQ_code SNQ_GetQueue (int qid);
SNQ_code SNQ_DumpQueue (QUEUE_Element **table, int *num_active);
SNQ_code SNQ_PrintQueue ();
SNQ_code SNQ_Wait ();
SNQ_code SNQ_Signal ();
char *SNQ_MakeFilename (int qid);

#endif




