/*
** Modality Worklist Database
**
** Author: 	Sandra von Gehlen
**		Oldenburg University, Germany
** Created:	03/1996
**
** Last Update:		hewett
** Update Date:		1997/01/13 15:37:07
** Source File:		/share/dicom/cvs-depot/dcmtk/wlistctn/include/wrklstdb.h,v
** CVS/RCS Revision:	1.6
** Status:		Exp
**
** CVS/RCS Log at end of file
*/
/*
** wrklstdb.h
*/

#ifndef WRKLSTDB_H
#define WRKLSTDB_H

#include "osconfig.h"

#include <stdio.h>
#include <string.h>

/* This #if code is suggested by the gnu autoconf documentation */
#if HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMELEN(dirent) (dirent)->d_namlen
# if HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# if HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# if HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif

#include "dctk.h"
#include "dcdebug.h"
#include "dcvm.h"
#include "dcstack.h"
#include "dclist.h"
#include "dcvrat.h"


enum WDBStatus {
    WDB_Refused_OutOfResorces = 0xa700,
    WDB_Failed_IdentifierDoesNotMatchSOPClass = 0xa900,
    WDB_Failed_UnableToProcess = 0xc000,
    WDB_Cancel = 0xfe00,
    WDB_Success = 0x0000,
    WDB_Pending = 0xff00,
    WDB_Pending_Warning = 0xff01
};

class WorklistDB
{
private:
    const char *dirpath;
    DIR *dirp;
    DcmDataset *identifiers;
    DcmList *objlist;
    DcmAttributeTag *errorElements;
    DcmAttributeTag *offendingElements;
    DcmLongString *errorComment;
    int fd;
    BOOL foundUnsupportedOptionalKey;

    void clearObjectList(void);

    void clearDataset(DcmDataset* identifiers);
    
    void putOffendingElements(DcmTag& tag);
    
    void putErrorElements(DcmTag& tag);

    BOOL checkIdentifiers(DcmDataset* identifiers);
    BOOL checkMatchingKey(DcmElement *elem);

    BOOL checkWklist(DcmDataset* dsetfile, const char* fname);

    int set_readlock(const char *path);

    int release_readlock();


public:

    WorklistDB(const char* dbDirectoryPath);

    ~WorklistDB();
    
    WDBStatus startFindRequest(DcmDataset& findRequestIdentifiers);

    WDBStatus nextFindResponse(DcmDataset& findResponseIdentifiers);

    WDBStatus cancelFindRequest();
    
    const DcmAttributeTag* getOffendingElements();
    
    const DcmLongString* getErrorComments();
};


#endif

/*
** CVS Log
** wrklstdb.h,v
** Revision 1.6  1997/01/13 15:37:07  hewett
** Added function to perform error checking of maching keys prior
** to a database search.
**
** Revision 1.5  1996/05/02 13:40:56  hewett
** Revised error handling.
**
** Revision 1.4  1996/04/30 13:59:24  savonge
** - more errors corrected
** - added checking of wklist database
**
** Revision 1.3  1996/04/29 19:13:45  hewett
** Added header comment and conditional inclusion of <dirent.h>.
**
**
*/
